package com.blzn.blefi.ui;

import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothGatt;
import android.bluetooth.BluetoothGattCallback;
import android.bluetooth.BluetoothGattCharacteristic;
import android.bluetooth.BluetoothGattService;
import android.bluetooth.BluetoothProfile;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.graphics.Color;
import android.os.Bundle;
import android.os.Handler;
import android.text.TextUtils;
import android.util.Log;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;
import androidx.recyclerview.widget.RecyclerView;

import com.blzn.ble.R;
import com.blzn.blefi.app.BaseActivity;
import com.blzn.blefi.app.BleLog;
import com.blzn.blefi.bus.EventMessage;
import com.blzn.blefi.constants.BleConstants;
import com.blzn.blefi.constants.BleUUIDParameter;
import com.blzn.blefi.constants.SettingsConstants;
import com.blzn.blefi.operation.OperationActivity;
import com.blzn.blefi.ui.wight.CustomDialog;
import com.blzn.blefi.ui.wight.FloatView;
import com.blzn.blefi.ui.wight.ToastUtils;
import com.blzn.blefi.utils.ClsUtils;
import com.example.hellojni.BlufiApp;

import org.greenrobot.eventbus.EventBus;

import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;
import java.util.Locale;
import java.util.UUID;

import blufi.ble.cb.BleCallback;
import blufi.ble.BleClient;
import blufi.ble.cb.BleException;
import blufi.ble.cb.BleNotifyCallback;
import blufi.ble.cb.BleWriteCallback;
import blufi.ble.params.ServiceUuid;

import com.blzn.blefi.resultBean.WifiInfo;

import blufi.ble.params.BlufiConfigureParams;
import blufi.ble.response.BleVersionResponse;

import com.blzn.blefi.resultBean.wifiState;
import com.google.gson.Gson;

import blufi.ble.params.BlbleParameter.Type.*;

import static com.blzn.blefi.bus.EventCode.A;

public class BleActivity extends BaseActivity {

    private static final int REQUEST_CONFIGURE_WIFISTA = 0x10;
    private static final int REQUEST_CONFIGURE = 0x20;
    private static final int REQUEST_DATESHOW = 0x30;
    private static final String TAG = "BleActivity";

    private final BleLog mLog = new BleLog(getClass());

    private BluetoothDevice mDevice;
    private BluetoothGatt mgatt;
    private BleClient mBleClient;
    private volatile boolean mConnected;

    private RecyclerView mMsgRecyclerView;
    private List<Message> mMsgList;
    private MsgAdapter mMsgAdapter;

    private Button mBlufiConnectBtn;
    private Button mBlufiDisconnectBtn;
    private Button mBlufiSecurityBtn;
    private Button mBlufiVersionBtn;
    private Button mBlufiConfigureBtn;
    private Button mBlufiDeviceStatusBtn;
    private Button mBlufiDeviceScanBtn;
    private Button mBlufiCustomBtn;
    private Button mBle_device_ota;
    private Button ble_CMD_ZB_RESET;
    private Button ble_DATA_ZB_LINKKEY;
    private Button ble_pressure;

    private String uuidPreference =
            (String) BlufiApp.getInstance().settingsGet(SettingsConstants.PREF_SETTINGS_KEY_SERVICE_UUID,
                    BleUUIDParameter.UUID_SERVICE.toString());

    private Handler handler = new Handler();
    private FloatView view;
    private boolean do_ble_pressure;
    private CustomDialog.Builder builder;
    private int testWriteBytes;

    private Runnable task = new Runnable() {
        public void run() {
        }
    };

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.blufi_activity);
        setHomeAsUpEnable(true);

        mDevice = getIntent().getParcelableExtra(BleConstants.KEY_BLE_DEVICE);
        assert mDevice != null;
        String deviceName = mDevice.getName() == null ? getString(R.string.string_unknown) :
                mDevice.getName();
        setTitle(deviceName);

        mMsgRecyclerView = findViewById(R.id.recycler_view);
        mMsgList = new LinkedList<>();
        mMsgAdapter = new MsgAdapter();
        mMsgRecyclerView.setAdapter(mMsgAdapter);

        BlufiButtonListener clickListener = new BlufiButtonListener();

        mBlufiConnectBtn = findViewById(R.id.blufi_connect);
        mBlufiConnectBtn.setOnClickListener(clickListener);
        mBlufiConnectBtn.setOnLongClickListener(clickListener);

        mBlufiDisconnectBtn = findViewById(R.id.blufi_disconnect);
        mBlufiDisconnectBtn.setOnClickListener(clickListener);
        mBlufiDisconnectBtn.setOnLongClickListener(clickListener);
        mBlufiDisconnectBtn.setEnabled(false);

        mBlufiSecurityBtn = findViewById(R.id.blufi_security);
        mBlufiSecurityBtn.setOnClickListener(clickListener);
        mBlufiSecurityBtn.setOnLongClickListener(clickListener);
        mBlufiSecurityBtn.setEnabled(false);

        mBlufiVersionBtn = findViewById(R.id.blufi_version);
        mBlufiVersionBtn.setOnClickListener(clickListener);
        mBlufiVersionBtn.setOnLongClickListener(clickListener);
        mBlufiVersionBtn.setEnabled(false);

        mBlufiConfigureBtn = findViewById(R.id.blufi_configure);
        mBlufiConfigureBtn.setOnClickListener(clickListener);
        mBlufiConfigureBtn.setOnLongClickListener(clickListener);
        mBlufiConfigureBtn.setEnabled(false);

        mBlufiDeviceScanBtn = findViewById(R.id.blufi_device_scan);
        mBlufiDeviceScanBtn.setOnClickListener(clickListener);
        mBlufiDeviceScanBtn.setOnLongClickListener(clickListener);
        mBlufiDeviceScanBtn.setEnabled(false);

        mBlufiDeviceStatusBtn = findViewById(R.id.blufi_device_status);
        mBlufiDeviceStatusBtn.setOnClickListener(clickListener);
        mBlufiDeviceStatusBtn.setOnLongClickListener(clickListener);
        mBlufiDeviceStatusBtn.setEnabled(false);

        mBlufiCustomBtn = findViewById(R.id.blufi_custom);
        mBlufiCustomBtn.setOnClickListener(clickListener);
        mBlufiCustomBtn.setOnLongClickListener(clickListener);
        mBlufiCustomBtn.setEnabled(false);


        mBle_device_ota = findViewById(R.id.ble_device_ota);
        mBle_device_ota.setOnClickListener(clickListener);
        mBle_device_ota.setOnLongClickListener(clickListener);
        mBle_device_ota.setEnabled(false);

        ble_CMD_ZB_RESET = findViewById(R.id.ble_CMD_ZB_RESET);
        ble_CMD_ZB_RESET.setOnClickListener(clickListener);
        ble_CMD_ZB_RESET.setEnabled(false);

        ble_DATA_ZB_LINKKEY = findViewById(R.id.ble_DATA_ZB_LINKKEY);
        ble_DATA_ZB_LINKKEY.setOnClickListener(clickListener);
        ble_DATA_ZB_LINKKEY.setEnabled(false);

        ble_pressure = findViewById(R.id.ble_pressure);
        ble_pressure.setOnClickListener(clickListener);
        ble_pressure.setEnabled(false);

//        handler.postDelayed(task,10000);//延迟调用 用于测试
//        handler.postDelayed(() -> {
//            handler.removeCallbacks(task);
//        },40000);

        IntentFilter filter = new IntentFilter("android.bluetooth.adapter.action.STATE_CHANGED");
        filter.addAction("android.bluetooth.device.action.PAIRING_REQUEST");
        filter.setPriority(Integer.MAX_VALUE);
//        registerReceiver(new BluetoothConnectActivityReceiver(), filter);
        connect();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        handler.removeCallbacks(reconnect_runnable);
        if (view != null) {
            view.removeWindow();
        }
        if (mBleClient != null) {
            mBleClient.close();
            mBleClient = null;
        }
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        Log.d("onActivityResult", "requestCode:" + String.valueOf(requestCode));
        Log.d("onActivityResult", "resultCode:" + String.valueOf(resultCode));
        if (requestCode == REQUEST_CONFIGURE_WIFISTA) {
            if (resultCode == RESULT_OK) {
                BlufiConfigureParams params =
                        (BlufiConfigureParams) data.getSerializableExtra(BleConstants.KEY_CONFIGURE_PARAM);
                // 获取配置信息
                assert params != null;
                Log.d("onActivityResult", "params:" + params.getStaPassword());
                Toast.makeText(this, "正在传输密码", Toast.LENGTH_LONG).show();
                //先停止当前的线程
                configure(params);
            } else {
                //停止线程
                mBleClient.stopTask();
            }
        }
        super.onActivityResult(requestCode, resultCode, data);
    }

    private void updateMessage(String message, boolean isNotificaiton) {
        runOnUiThread(() -> {
            Message msg = new Message();
            msg.text = message;
            msg.isNotification = isNotificaiton;
            mMsgList.add(msg);
            mMsgAdapter.notifyItemInserted(mMsgList.size() - 1);
            mMsgRecyclerView.scrollToPosition(mMsgList.size() - 1);
        });
    }

    /**
     * Try to connect device
     */
    private void connect() {
        mBlufiConnectBtn.setEnabled(false);

        if (mBleClient != null) {
            mBleClient.close();
            mBleClient = null;
        }

        mBleClient = new BleClient(getApplicationContext(), mDevice, uuidPreference,
                new ServiceUuid().general(
                UUID.fromString(String.valueOf(BleUUIDParameter.getUUID().get(uuidPreference).getUUID_SERVICE().toString())),
                UUID.fromString(String.valueOf(BleUUIDParameter.getUUID().get(uuidPreference).getUUID_WRITE_CHARACTERISTIC().toString())),
                UUID.fromString(String.valueOf(BleUUIDParameter.getUUID().get(uuidPreference).getUUID_NOTIFICATION_CHARACTERISTIC().toString()))
        ));
        mBleClient.setGattCallback(new GattCallback());
        mBleClient.setBleCallback(new BleCallbackMain());
        mBleClient.connect();
        if ((boolean) BlufiApp.getInstance().settingsGet(SettingsConstants.SETTINGS_KEY_SHOW_SPEED_FRAME,
                false)) {
            view = new FloatView(this);
            view.show();
        }
    }

    /**
     * Request device disconnect the connection.
     */
    private void disconnect() {
        Toast.makeText(this, R.string.disconnecting_wifi, Toast.LENGTH_LONG).show();
        if (mBleClient != null) {
            mBleClient.requestCloseConnection();
        }
    }

    /**
     * If negotiate security success, the continue communication data will be encrypted.
     */
    private void negotiateSecurity() {
        mBleClient.negotiateSecurity();
    }

    /**
     * Request to configure station or softap
     *
     * @param params configure params
     */
    private void configure(BlufiConfigureParams params) {
        mBleClient.configure(params);
    }

    /**
     * Request to get device current status
     */
    private void requestDeviceStatus() {
        mBleClient.requestDeviceStatus();
    }

    /**
     * Request to get device blufi version
     */
    private void requestDeviceVersion() {
        mBleClient.requestDeviceVersion();
    }

    /**
     * Request to get AP list that the device scanned
     */
    private void requestDeviceWifiScan() {
        mBleClient.requestDeviceWifiScan();
        Intent intent = new Intent();
        intent.setClass(this, DateResultActivity.class);
        startActivityForResult(intent, REQUEST_CONFIGURE_WIFISTA);
    }

    /**
     * Try to post custom data
     */
    private void postCustomData() {
        new AlertDialog.Builder(this)
                .setTitle(R.string.blufi_custom_dialog_title)
                .setView(R.layout.blufi_custom_data_dialog)
                .setPositiveButton(android.R.string.ok, (dialog, which) -> {
                    EditText editText = ((AlertDialog) dialog).findViewById(R.id.edit_text);
                    assert editText != null;
                    String dataStr = editText.getText().toString();
                    if (!TextUtils.isEmpty(dataStr)) {
                        mBleClient.postCustomData(dataStr.getBytes());
                    }
                })
                .show();
    }

    private void onGattConnected() {
        mConnected = true;
        runOnUiThread(() -> {
            mBlufiConnectBtn.setEnabled(false);
//            mBlufiDisconnectBtn.setEnabled(true);
        });
    }

    private void onGattServiceCharacteristicDiscovered() {
        runOnUiThread(() -> {
            mBlufiSecurityBtn.setEnabled(true);
            mBlufiVersionBtn.setEnabled(true);
            mBlufiConfigureBtn.setEnabled(true);
            mBlufiDeviceStatusBtn.setEnabled(true);
            mBlufiDeviceScanBtn.setEnabled(true);
            mBlufiCustomBtn.setEnabled(true);
            mBle_device_ota.setEnabled(true);
            ble_CMD_ZB_RESET.setEnabled(true);
            ble_DATA_ZB_LINKKEY.setEnabled(true);
            ble_pressure.setEnabled(true);

        });
    }

    private void onGattDisconnected() {
        mConnected = false;
        runOnUiThread(() -> {
            mBlufiConnectBtn.setEnabled(true);

            mBlufiDisconnectBtn.setEnabled(false);
            mBlufiSecurityBtn.setEnabled(false);
            mBlufiVersionBtn.setEnabled(false);
            mBlufiConfigureBtn.setEnabled(false);
            mBlufiDeviceStatusBtn.setEnabled(false);
            mBlufiDeviceScanBtn.setEnabled(false);
            mBlufiCustomBtn.setEnabled(false);
            mBle_device_ota.setEnabled(false);
            ble_CMD_ZB_RESET.setEnabled(false);
            ble_DATA_ZB_LINKKEY.setEnabled(false);
            ble_pressure.setEnabled(false);

        });
    }

    private Runnable reconnect_runnable = new Runnable() {
        @Override
        public void run() {
            BleClient.getInstance().connect();
            handler.postDelayed(reconnect_runnable, 1500);
        }
    };

    /**
     * mBleClient call onCharacteristicWrite and onCharacteristicChanged is required
     */
    private class GattCallback extends BluetoothGattCallback {
        @Override
        public void onConnectionStateChange(BluetoothGatt gatt, int status, int newState) {
            if (status != 0) {
                gatt.close();
            }
            String devAddr = gatt.getDevice().getAddress();
            mLog.d(String.format(Locale.ENGLISH, "onConnectionStateChange addr=%s, status=%d, newState=%d",
                    devAddr, status, newState));
            if (status == BluetoothGatt.GATT_SUCCESS) {
                switch (newState) {
                    case BluetoothProfile.STATE_CONNECTED:
                        onGattConnected();
                        updateMessage(String.format("Connected %s", devAddr), false);
//                        gatt.requestMtu(200);
                        handler.removeCallbacks(reconnect_runnable);

                        break;
                    case BluetoothProfile.STATE_DISCONNECTED:
                        gatt.close();
                        onGattDisconnected();
                        updateMessage(String.format("Disconnected %s", devAddr), false);
                        break;
                }
            } else {
                // TODO: 2021/8/24 待处理133 257 
                gatt.close();
                onGattDisconnected();
                updateMessage(String.format(Locale.ENGLISH, "Disconnect %s, status=%d", devAddr, status),
                        false);
                if ((boolean) BlufiApp.getInstance().settingsGet(SettingsConstants.PREF_SETTINGS_KEY_AUTO_CONNECT, false)) {
                    handler.postDelayed(reconnect_runnable, 1000);
                }
            }
        }

        @Override
        public void onMtuChanged(BluetoothGatt gatt, int mtu, int status) {
            mLog.d(String.format(Locale.ENGLISH, "onMtuChanged status=%d, mtu=%d", status, mtu));
            if (status == BluetoothGatt.GATT_SUCCESS) {
                updateMessage(String.format(Locale.ENGLISH, "Set mtu complete, mtu=%d ", mtu), false);
                mBleClient.setMtu(mtu);
                handler.postDelayed(new Runnable() {
                    @Override
                    public void run() {
//                        requestDeviceVersion();
                    }
                }, 500);

            } else {
                updateMessage(String.format(Locale.ENGLISH, "Set mtu failed, mtu=%d, status=%d", mtu,
                        status), false);
            }

//            onGattServiceCharacteristicDiscovered();
        }

        @Override
        public void onServicesDiscovered(BluetoothGatt gatt, int status) {
            mLog.d(String.format(Locale.ENGLISH, "onServicesDiscovered status=%d", status));
            if (status != BluetoothGatt.GATT_SUCCESS) {
                gatt.disconnect();
                updateMessage(String.format(Locale.ENGLISH, "Discover services error status %d", status),
                        false);
            }
        }

        @Override
        public void onCharacteristicWrite(BluetoothGatt gatt, BluetoothGattCharacteristic characteristic,
                                          int status) {
            if (status != BluetoothGatt.GATT_SUCCESS) {
                gatt.disconnect();
                updateMessage(String.format(Locale.ENGLISH, "WriteChar error status %d", status), false);
            }
        }

        @Override
        public void onCharacteristicRead(BluetoothGatt gatt, BluetoothGattCharacteristic characteristic,
                                         int status) {

            final byte[] data = characteristic.getValue();
            Log.d("onCharacteristicRead", Arrays.toString(data));
        }
    }

    private class BleCallbackMain extends BleCallback {
        @Override
        public void onGattPrepared(BleClient client, BluetoothGatt gatt, BluetoothGattService service,
                                   BluetoothGattCharacteristic writeChar,
                                   BluetoothGattCharacteristic notifyChar) {
            mgatt = gatt;
//            int mtu = BleConstants.MAX_MTU_LENGTH;
            int mtuLen =
                    (int) BlufiApp.getInstance().settingsGet(SettingsConstants.PREF_SETTINGS_KEY_MTU_LENGTH
                            , BleConstants.MAX_MTU_LENGTH);
//            mtu = 527;
//            if (Build.VERSION.SDK_INT == Build.VERSION_CODES.Q
//                    && Build.MANUFACTURER.toLowerCase().startsWith("samsung")) {
//                mtu = 24;
//            }
            boolean requestMtu = gatt.requestMtu(mtuLen);
            if (!requestMtu) {
                mLog.w("Request mtu failed");
                updateMessage(String.format(Locale.ENGLISH, "Request mtu %d failed", mtuLen), false);
            }

            List<BluetoothGattService> bluetoothGattServiceLists = gatt.getServices();
            for (BluetoothGattService bluetoothGattService : bluetoothGattServiceLists) {
                updateMessage(bluetoothGattService.getUuid().toString(), true);
            }

            if (service == null) {
                mBlufiConfigureBtn.setEnabled(true);
                mBlufiVersionBtn.setEnabled(false);
                mBlufiDeviceStatusBtn.setEnabled(false);
                mBlufiDeviceScanBtn.setEnabled(false);
                mBlufiCustomBtn.setEnabled(false);
                mBle_device_ota.setEnabled(false);
                ble_CMD_ZB_RESET.setEnabled(false);
                ble_DATA_ZB_LINKKEY.setEnabled(false);
                ble_pressure.setEnabled(false);
                return;
            } else {
                onGattServiceCharacteristicDiscovered();
            }

            if (writeChar == null) {
                mLog.w("Get write characteristic failed");
//                gatt.disconnect();
                updateMessage("Get wifiList write characteristic failed", false);
//                return;
            }
            if (notifyChar == null) {
                mLog.w("Get notification characteristic failed");
                //gatt.disconnect();
                updateMessage("Get wifiList notification characteristic failed", false);
                //return;
            }

            updateMessage("Discover service and characteristics success", false);

        }

        @Override
        public void onNegotiateSecurityResult(BleClient client, int status) {
            if (status == STATUS_SUCCESS) {
                updateMessage("Negotiate security complete", false);
            } else {
                updateMessage("Negotiate security failed， code=" + status, false);
            }

            mBlufiSecurityBtn.setEnabled(mConnected);
        }

        @Override
        public void onConfigureResult(BleClient client, int status) {
            if (status == STATUS_SUCCESS) {
                updateMessage("Post configure params complete", false);
//                Log.d("onConfigureResult", "准备查阅状态:requestDeviceStatus");
                handler.postDelayed(new Runnable() {
                    @Override
                    public void run() {
                        requestDeviceStatus();
                    }
                }, 2000);
            } else if (status == BleCallback.CODE_NEG_ERR_SECURITY) {
                updateMessage("设备已配对", false);
            } else {
                updateMessage("Post configure params failed, code=" + status, false);
            }

            mBlufiConfigureBtn.setEnabled(mConnected);
        }

        @Override
        public void onDeviceStatusResponse(BleClient client, int status, String wifiStateStr) {
            if (status == 10010) {
                if (wifiStateStr == null || wifiStateStr.isEmpty()) {
                    return;
                }
                Gson gson = new Gson();
                wifiState wifiState = gson.fromJson(wifiStateStr, wifiState.class);
                switch (wifiState.getState()) {
                    case WIFI_STATE_ENUM_LIST
                            .WIFI_STATE_CONNECTED_IP_GOT:
                        updateMessage("WIFI CONNECTED IP: " + wifiState.getIp()
                                , true);
                        ToastUtils.show(BleActivity.this, "连接成功");
                        mBlufiDisconnectBtn.setEnabled(true);
                        break;
                    case WIFI_STATE_ENUM_LIST
                            .WIFI_STATE_CONNECTING:
                        ToastUtils.show(BleActivity.this, "wifi连接中");
                        handler.postDelayed(new Runnable() {
                            @Override
                            public void run() {
                                requestDeviceStatus();
                            }
                        }, 500);
                        break;

                    case WIFI_STATE_ENUM_LIST
                            .WIFI_STATE_IDLE:
                        updateMessage("设备wifi断开", true);
                        mBlufiDisconnectBtn.setEnabled(false);
                        break;

                    case WIFI_STATE_ENUM_LIST
                            .WIFI_STATE_CONNECTED_IP_GETTING:
                        ToastUtils.show(BleActivity.this, "设备获取ip地址中");

                        handler.postDelayed(new Runnable() {
                            @Override
                            public void run() {
                                requestDeviceStatus();
                            }
                        }, 300);
                        break;

                    case WIFI_STATE_ENUM_LIST
                            .WIFI_STATE_PSK_ERROR:
                        updateMessage("设备wifi密码错误", true);
                        break;

                    case WIFI_STATE_ENUM_LIST
                            .WIFI_STATE_NO_AP_FOUND:
                        updateMessage("设备未找到该wifi", true);
                        break;

                    default:
                        updateMessage("未知状态，请稍后手动更新", true);
                        break;

                }

            } else {
                updateMessage("Device status response error, code=" + status, false);
            }

            mBlufiDeviceStatusBtn.setEnabled(mConnected);
        }

        @Override
        public void onDeviceScanResult(BleClient client, int status, String resultsStr) {
            if (status == 10010) {
                if (resultsStr == null || resultsStr.isEmpty()) {
                    return;
                }
                Gson gson = new Gson();
                WifiInfo shopInfo = gson.fromJson(resultsStr, WifiInfo.class);
                if (shopInfo.getSsid() != null) {
                    if (!shopInfo.getSsid().isEmpty()) {
                        EventMessage msg = new EventMessage(A, shopInfo);
                        EventBus.getDefault().post(msg);
                    }
                }

            } else {
                updateMessage("Device scan result error, code=" + status, false);
            }
        }

        @Override
        public void onDeviceVersionResponse(BleClient client, int status, BleVersionResponse response) {
            if (status == STATUS_SUCCESS) {
                updateMessage(String.format("Receive device version: %s", response.getVersionString()),
                        true);
            } else {
                updateMessage("Device version error, code=" + status, false);
            }

            mBlufiVersionBtn.setEnabled(mConnected);
        }

        @Override
        public void onPostCustomDataResult(BleClient client, int status, String dataStr) {
            String format = "Post data %s %s";
            if (status == STATUS_SUCCESS) {
//                updateMessage(String.format(format, dataStr, "complete"), false);
            } else {
//                updateMessage(String.format(format, dataStr, "failed"), false);
            }
        }

        @Override
        public void onReceiveCustomData(BleClient client, int status, byte[] data) {
            if (status == STATUS_SUCCESS) {
                String customStr = new String(data);
                updateMessage(String.format("Receive custom data:\n%s", customStr), true);
            } else {
                updateMessage("Receive custom data error, code=" + status, false);
            }
        }

        @Override
        public void onError(BleClient client, int errCode) {

            switch (errCode) {
                case CODE_WRITE_DATA_FAILED://读取出错超过十次

                    break;
                default:
                    break;
            }
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    updateMessage(String.format(Locale.ENGLISH, "Receive error code %d", errCode), false);
                }
            });
        }

        @Override
        public void onThroughput(int num, int read_data_size) {
//            Log.w(TAG,"num:"+num);
//            Log.w(TAG,"read_data_size:"+read_data_size);

            handler.post(new Runnable() {
                @Override
                public void run() {
                    if (view != null) {
                        view.tv_show.setText(
                                "get:\u3000" + (float) (read_data_size / (1 * 1)) + "B/s \n" +
                                        "send: " + (float) (num / (1 * 1)) + " B/s "
                        );
                    }
                    if (builder != null) {
                        builder.setSpeed(num, read_data_size);
//                    builder.setAccept(10,10,new byte[10]);
                    }
                }
            });
        }
    }

    private class BlufiButtonListener implements View.OnClickListener, View.OnLongClickListener {
        private Toast mToast;

        @Override
        public void onClick(View v) {
            if (v == mBlufiConnectBtn) {
                Toast.makeText(BleActivity.this, "正在连接中", Toast.LENGTH_LONG).show();
                connect();
            } else if (v == mBlufiDisconnectBtn) {
                //发送断开wifi的指令
                disconnect();
                handler.postDelayed(new Runnable() {
                    @Override
                    public void run() {
                        requestDeviceStatus();
                    }
                }, 1300);
            } else if (v == mBlufiSecurityBtn) {
                negotiateSecurity();
            } else if (v == mBlufiConfigureBtn) {
                // go to wifi config
                //configureOptions();
                // will go2 detail
                startActivity(new Intent(BleActivity.this, OperationActivity.class)
//                        .putExtra(KEY_DATA, mBleClient)
                );
            } else if (v == mBlufiDeviceScanBtn) {
                requestDeviceWifiScan();
            } else if (v == mBlufiVersionBtn) {
                requestDeviceVersion();
                handler.postDelayed(new Runnable() {
                    @Override
                    public void run() {
//                        requestDeviceVersion();
                    }
                }, 100);
            } else if (v == mBlufiDeviceStatusBtn) {
                requestDeviceStatus();
            } else if (v == mBlufiCustomBtn) {
                postCustomData();
            } else if (v == mBle_device_ota) {
                startActivity(new Intent(BleActivity.this, OtaActivity.class));
            } else if (v == ble_CMD_ZB_RESET) {
                send_ble_CMD_ZB_RESET();
            } else if (v == ble_DATA_ZB_LINKKEY) {
                send_ble_DATA_ZB_LINKKEY();
            } else if (v == ble_pressure) {
//                if (ble_pressure.getText().equals("关闭测试")){
//                    ble_pressure.setText("压力测试");
//                    do_ble_pressure =false;
//                    updateMessage(String.format(Locale.ENGLISH, "stop_ble_pressure"), false);
//                    return;
//                }
//                ble_pressure.setText("关闭测试");
                testWriteBytes = 0;
                builder = new CustomDialog.Builder(BleActivity.this);
                builder.setPositiveButton("发送", new CustomDialog.ClickListener() {
                    @Override
                    public void onClick(CustomDialog dialog, String s, int i, int buttonPositive) {
                        if (i == -1) {
                            do_ble_pressure = false;
                        } else {
                            do_ble_pressure = true;
                        }
                        start_ble_pressure(s, i);
                    }

                    @Override
                    public void onStop() {
                        do_ble_pressure = false;
                    }

                    @Override
                    public void onRead(boolean b) {
                        if (b) {
                            ToastUtils.show(getApplication(), "开始监听");
                            bleNotifyCallback.setKey(BleUUIDParameter.getUUID().get(uuidPreference).getUUID_NOTIFICATION_CHARACTERISTIC().toString());
                            BleClient.getInstance().notify(
                                    BleClient.getInstance().getGatt().getDevice(),
                                    BleUUIDParameter.getUUID().get(uuidPreference).getUUID_SERVICE().toString(),
                                    BleUUIDParameter.getUUID().get(uuidPreference).getUUID_NOTIFICATION_CHARACTERISTIC().toString(),
                                    bleNotifyCallback
                            );
                        } else {
                            ToastUtils.show(getApplication(), "关闭监听");
                            BleClient.getInstance().stopNotify(
                                    BleClient.getInstance().getGatt().getDevice(),
                                    BleUUIDParameter.getUUID().get(uuidPreference).getUUID_SERVICE().toString(),
                                    BleUUIDParameter.getUUID().get(uuidPreference).getUUID_NOTIFICATION_CHARACTERISTIC().toString());
                        }
                    }
                });

                builder.setNegativeButton("关闭", new DialogInterface.OnClickListener() {
                            @Override
                            public void onClick(DialogInterface dialogInterface, int i) {
                                dialogInterface.dismiss();
                                BleClient.getInstance().stopNotify(
                                        BleClient.getInstance().getGatt().getDevice(),
                                        BleUUIDParameter.getUUID().get(uuidPreference).getUUID_SERVICE().toString(),
                                        BleUUIDParameter.getUUID().get(uuidPreference).getUUID_NOTIFICATION_CHARACTERISTIC().toString());
                            }
                        }
                );
                builder.create().show();
                updateMessage(String.format(Locale.ENGLISH, "start_ble_pressure"), false);

            }
        }

        private void start_ble_pressure(String value, int interval) {
            byte[] bytes = new byte[mBleClient.getMtu()];
            Arrays.fill(bytes, (byte) 8);//把第3+1位（包括）到4+1位（不包括）的数替换成88
            handler.post(new Runnable() {
                @Override
                public void run() {

                }
            });
            new Thread(new Runnable() {
                @Override
                public void run() {
//                        mBleClient.postCustomData(bytes);
                    BleClient.getInstance().write(
                            BleClient.getInstance().getGatt().getDevice(),
                            BleUUIDParameter.getUUID().get(uuidPreference).getUUID_SERVICE().toString(),
                            BleUUIDParameter.getUUID().get(uuidPreference).getUUID_WRITE_CHARACTERISTIC().toString(),
                            value.getBytes(),
                            new BleWriteCallback() {
                                @Override
                                public void onWriteSuccess(final int current, final int total,
                                                           final byte[] justWrite) {
//                                        Log.d(TAG,"onWriteSuccess");
                                    testWriteBytes += justWrite.length;
                                    runOnUiThread(() -> {
                                        builder.setWriteBytes(testWriteBytes);
                                    });
                                    if (do_ble_pressure) {
                                        handler.postDelayed(new Runnable() {
                                            @Override
                                            public void run() {
                                                start_ble_pressure(value, interval);
                                            }
                                        }, interval);
                                    }
                                }

                                @Override
                                public void onWriteFailure(final BleException exception) {
                                    Log.e(TAG, "onWriteFailure");
                                    if (do_ble_pressure) {
                                        handler.postDelayed(new Runnable() {
                                            @Override
                                            public void run() {
                                                start_ble_pressure(value, interval);
                                            }
                                        }, interval);
                                    }
                                    runOnUiThread(() -> {
//                                        ToastUtils.show(BleActivity.this,exception.toString());
                                    });
                                }
                            });
                }
            }).start();
        }

        private BleNotifyCallback bleNotifyCallback = new BleNotifyCallback() {
            @Override
            public void onNotifySuccess() {
            }

            @Override
            public void onNotifyFailure(final BleException exception) {
            }

            @Override
            public void onCharacteristicChanged(byte[] data) {
                if (builder != null) {
                    handler.post(new Runnable() {
                        @Override
                        public void run() {
                            builder.refreshLogView(data);
                        }
                    });
                }
            }
        };

        @Override
        public boolean onLongClick(View v) {
            if (mToast != null) {
                mToast.cancel();
            }

            int msgRes = 0;
            if (v == mBlufiConnectBtn) {
                msgRes = R.string.blufi_function_connect_msg;
            } else if (v == mBlufiDisconnectBtn) {
                msgRes = R.string.blufi_function_disconnect_msg;
            } else if (v == mBlufiSecurityBtn) {
                msgRes = R.string.blufi_function_security_msg;
            } else if (v == mBlufiConfigureBtn) {
                msgRes = R.string.blufi_function_configure_msg;
            } else if (v == mBlufiDeviceScanBtn) {
                msgRes = R.string.blufi_function_device_scan_msg;
            } else if (v == mBlufiVersionBtn) {
                msgRes = R.string.blufi_function_version_msg;
            } else if (v == mBlufiDeviceStatusBtn) {
                msgRes = R.string.blufi_function_device_status_msg;
            } else if (v == mBlufiCustomBtn) {
                msgRes = R.string.blufi_function_custom_msg;
            }

            mToast = Toast.makeText(BleActivity.this, msgRes, Toast.LENGTH_SHORT);
            mToast.show();

            return true;
        }
    }

    private void send_ble_DATA_ZB_LINKKEY() {
        mBleClient.send_ble_DATA_ZB_LINKKEY();
    }

    private void send_ble_CMD_ZB_RESET() {
        mBleClient.send_ble_CMD_ZB_RESET();
    }

    private class MsgHolder extends RecyclerView.ViewHolder {
        TextView text1;

        MsgHolder(View itemView) {
            super(itemView);
            text1 = itemView.findViewById(android.R.id.text1);
        }
    }

    private class MsgAdapter extends RecyclerView.Adapter<MsgHolder> {

        @NonNull
        @Override
        public MsgHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
            View view = getLayoutInflater().inflate(R.layout.blufi_message_item, parent, false);
            return new MsgHolder(view);
        }

        @Override
        public void onBindViewHolder(@NonNull MsgHolder holder, int position) {
            Message msg = mMsgList.get(position);
            holder.text1.setText(msg.text);
            holder.text1.setTextColor(msg.isNotification ? Color.RED : Color.BLACK);
        }

        @Override
        public int getItemCount() {
            return mMsgList.size();
        }
    }

    private class Message {
        String text;
        boolean isNotification;
    }

    public class BluetoothConnectActivityReceiver extends BroadcastReceiver {

        @Override
        public void onReceive(Context context, Intent intent) {
//            if (intent.getAction().equals("android.bluetooth.device.action.PAIRING_REQUEST")) {
//                BluetoothDevice mBluetoothDevice = intent.getParcelableExtra(BluetoothDevice.EXTRA_DEVICE);
//                try {
//                    //(三星)4.3版本测试手机还是会弹出用户交互页面(闪一下)，如果不注释掉下面这句页面不会取消但可以配对成功。(中兴，魅族4(Flyme 6))5.1版本手机两中情况下都正常
//                    //ClsUtils.setPairingConfirmation(mBluetoothDevice.getClass(), mBluetoothDevice, true);
//                    abortBroadcast();//如果没有将广播终止，则会出现一个一闪而过的配对框。
//                    //3.调用setPin方法进行配对...
//                    boolean ret = ClsUtils.setPin(mBluetoothDevice.getClass(), mBluetoothDevice,
//                    "你需要设置的PIN码");
//                } catch (Exception e) {
//                    e.printStackTrace();
//                }
//            }else
            Log.d("BroadcastReceiver", intent.getAction());
            if (intent.getAction().equals("android.bluetooth.device.action.PAIRING_REQUEST")) {
                BluetoothDevice device = intent.getParcelableExtra(BluetoothDevice.EXTRA_DEVICE);
                Log.d("BroadcastReceiver", intent.getAction());
                try {
                    int mType = intent.getIntExtra(BluetoothDevice.EXTRA_PAIRING_VARIANT,
                            BluetoothDevice.ERROR);


//                    BluetoothDevice mBluetoothDevice = intent.getParcelableExtra(BluetoothDevice
//                    .EXTRA_DEVICE);
//                    try {
//                        //(三星)4.3版本测试手机还是会弹出用户交互页面(闪一下)，如果不注释掉下面这句页面不会取消但可以配对成功。(中兴，魅族4(Flyme 6))5
//                        .1版本手机两中情况下都正常
//                        //ClsUtils.setPairingConfirmation(mBluetoothDevice.getClass(), mBluetoothDevice,
//                        true);
//                        abortBroadcast();//如果没有将广播终止，则会出现一个一闪而过的配对框。
//                        //3.调用setPin方法进行配对...
//                        boolean ret = ClsUtils.setPin(mBluetoothDevice.getClass(), mBluetoothDevice,
//                        "123123");
//                    } catch (Exception e) {
//                        e.printStackTrace();
//                    }

                    Log.d("getIntExtra", String.valueOf(mType));
                    // 蓝牙有两种配对类型一种是pin，一种是密钥 先搞清楚你的设备是哪一种在做，不然瞎忙活
                    boolean isSuccess = false;
                    switch (mType) {
                        case 0:
                            // 反射 不会自己收ClsUtils
                            isSuccess = ClsUtils.setPin(device.getClass(), device, String.valueOf(123456));
                            break;
                        case 1:
                            // 这个方法是我自己加的 ,不会的可以照着setPin写一个setPasskey boolean
                            // Bluetooth.setPasskey(int)
                            isSuccess = ClsUtils.setPassKey(device.getClass(), device,
                                    String.valueOf(123456));
                    }

                    //配对不成功就弹出来
                    if (isSuccess) {
                        // 重点 先前已经设置我的广播接收者的优先者为最高 收到广播后 停止广播向下传递
                        // 因为系统的配对框的消失与弹出是通过广播做到的 我看源码得知 ，我们要将它
                        // 扼杀在摇篮中
                        abortBroadcast();
                        // 怕没拦截成功 补上一刀 发送一个关闭对话框的广播
                        context.sendBroadcast(new Intent("android.bluetooth.device.action.PAIRING_CANCEL"));
                    }

                } catch (Exception e) {
                    e.printStackTrace();
                }

            }
        }
    }
}
