package com.blzn.blefi.ui;

import android.Manifest;
import android.app.Activity;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.le.BluetoothLeScanner;
import android.bluetooth.le.ScanResult;
import android.bluetooth.le.ScanSettings;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.location.LocationManager;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.os.SystemClock;
import android.text.Spannable;
import android.text.SpannableStringBuilder;
import android.text.TextUtils;
import android.text.style.ForegroundColorSpan;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;
import androidx.core.app.ActivityCompat;
import androidx.core.location.LocationManagerCompat;
import androidx.recyclerview.widget.RecyclerView;
import androidx.swiperefreshlayout.widget.SwipeRefreshLayout;

import com.blzn.blefi.app.BleLog;
import com.blzn.blefi.constants.BleConstants;
import com.blzn.blefi.constants.SettingsConstants;
import com.blzn.ble.R;
import com.blzn.blefi.utils.WindowPermissionCheck;
import com.example.hellojni.BlufiApp;
import com.example.wifiapp.WifiApActivity;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;

import progressbar.com.progressbarlibrary.PgDialog;

import static com.blzn.blefi.utils.FileUtil.getPath;


public class MainActivity extends AppCompatActivity {
    private static final long TIMEOUT_SCAN = 4000L;

    private static final int REQUEST_PERMISSION = 0x01;
    private static final int REQUEST_BLUFI = 0x10;

    private static final int MENU_SETTINGS = 0x01;
    private static final int TO_AP = 0x02;


    private final BleLog mLog = new BleLog(getClass());

    private SwipeRefreshLayout mRefreshLayout;

    private RecyclerView mRecyclerView;
    private List<ScanResult> mBleList;
    private BleAdapter mBleAdapter;

    private Map<String, ScanResult> mDeviceMap;
    private ScanCallback mScanCallback;
    private String mBlufiFilter;
    private volatile long mScanStartTime;

    private ExecutorService mThreadPool;
    private Future mUpdateFuture;
    private static int REQUEST_ENABLE_BT = 5;

    private Handler handler = new Handler();

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        setContentView(R.layout.main_activity);
        Toolbar toolbar = findViewById(R.id.toolbar);
        //setSupportActionBar(toolbar);
        Toast.makeText(this, "自动扫描中", Toast.LENGTH_LONG).show();

        mThreadPool = Executors.newSingleThreadExecutor();

        mRefreshLayout = findViewById(R.id.refresh_layout);
        mRefreshLayout.setColorSchemeResources(R.color.colorAccent);
        mRefreshLayout.setOnRefreshListener(this::scan);

        mRecyclerView = findViewById(R.id.recycler_view);
        mBleList = new LinkedList<>();
        mBleAdapter = new BleAdapter();
        mRecyclerView.setAdapter(mBleAdapter);

        mDeviceMap = new HashMap<>();
        mScanCallback = new ScanCallback();
        byte ba = (byte) ((130) & 0xff);
        Log.v("log", String.valueOf(ba));

//        ActivityCompat.requestPermissions(this, new String[]{Manifest.permission.ACCESS_FINE_LOCATION},
//        REQUEST_PERMISSION);
//        ActivityCompat.requestPermissions(this, new String[]{Manifest.permission.BLUETOOTH_PRIVILEGED},
//        REQUEST_PERMISSION);

        String[] PERMISSIONS_STORAGE = {
                Manifest.permission.READ_EXTERNAL_STORAGE,
                Manifest.permission.WRITE_EXTERNAL_STORAGE,
                Manifest.permission.ACCESS_FINE_LOCATION,
                Manifest.permission.BLUETOOTH_PRIVILEGED};
        ActivityCompat.requestPermissions(this, PERMISSIONS_STORAGE, REQUEST_PERMISSION);

    }

    @Override
    protected void onResume() {
        super.onResume();
//        Log.d("TAG onResume", "线程id:"+android.os.Process.myTid());
        //scan();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        stopScan();
        mThreadPool.shutdownNow();
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions,
                                           @NonNull int[] grantResults) {
        int size = permissions.length;
        for (int i = 0; i < size; ++i) {
            String permission = permissions[i];
            int grant = grantResults[i];

            if (permission.equals(Manifest.permission.ACCESS_FINE_LOCATION)) {
                if (grant == PackageManager.PERMISSION_GRANTED) {
                    mRefreshLayout.setRefreshing(true);
                    scan();
                }
            }
        }
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        if (requestCode == REQUEST_BLUFI) {
            mRefreshLayout.setRefreshing(true);
            scan();
            return;
        } else if (requestCode == REQUEST_ENABLE_BT) {
            mRefreshLayout.setRefreshing(true);
            scan();
            return;
        }
        super.onActivityResult(requestCode, resultCode, data);
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        menu.add(Menu.NONE, MENU_SETTINGS, 0, R.string.main_menu_settings);
        menu.add(Menu.NONE, TO_AP, 0, R.string.main_menu_to_ap);

        return super.onCreateOptionsMenu(menu);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        final int itemId = item.getItemId();
        if (itemId == MENU_SETTINGS) {
            startActivity(new Intent(this, SettingsActivity.class));
            return true;
        } else if (itemId == TO_AP) {
            Intent intent = new Intent();
            intent.setClass(MainActivity.this, WifiApActivity.class);
            startActivity(intent);
            return true;

        }
        return super.onOptionsItemSelected(item);
    }

    private void scan() {
        Log.d("TAG scan", "scan:" + android.os.Process.myTid());
        BluetoothAdapter adapter = BluetoothAdapter.getDefaultAdapter();
        if (adapter == null) {
            // Device does not support Bluetooth
            Toast.makeText(this, "设备不支持", Toast.LENGTH_LONG).show();
            return;
        }

        if (!adapter.isEnabled()) {
            Intent enableBtIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
            startActivityForResult(enableBtIntent, REQUEST_ENABLE_BT);
            return;
        }


        BluetoothLeScanner scanner = adapter.getBluetoothLeScanner();
        if (!adapter.isEnabled() || scanner == null) {
            Toast.makeText(this, R.string.main_bt_disable_msg, Toast.LENGTH_SHORT).show();
            mRefreshLayout.setRefreshing(false);
            return;
        }

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            // Check location enable
            LocationManager locationManager = (LocationManager) getSystemService(LOCATION_SERVICE);
            boolean locationEnable =
                    locationManager != null && LocationManagerCompat.isLocationEnabled(locationManager);
            if (!locationEnable) {
                Toast.makeText(this, R.string.main_location_disable_msg, Toast.LENGTH_SHORT).show();
                mRefreshLayout.setRefreshing(false);
                return;
            }
        }

        mDeviceMap.clear();
        mBleList.clear();
        mBleAdapter.notifyDataSetChanged();
        mBlufiFilter =
                (String) BlufiApp.getInstance().settingsGet(SettingsConstants.PREF_SETTINGS_KEY_BLE_PREFIX,
                        BleConstants.BLUFI_PREFIX);
        mScanStartTime = SystemClock.elapsedRealtime();
        mScanCallback = new ScanCallback();
        mLog.d("Start scan ble");
        scanner.startScan(null,
                new ScanSettings.Builder().setScanMode(ScanSettings.SCAN_MODE_LOW_LATENCY).build(),
                mScanCallback);
        if (!mThreadPool.isShutdown()) {
            mUpdateFuture = mThreadPool.submit(() -> {
                while (!Thread.currentThread().isInterrupted()) {
                    try {
                        Thread.sleep(8000);
                    } catch (InterruptedException e) {
                        e.printStackTrace();
                        break;
                    }

                    long scanCost = SystemClock.elapsedRealtime() - mScanStartTime;
                    if (scanCost > TIMEOUT_SCAN) {
                        break;
                    }

                    onIntervalScanUpdate(false);
                }

                BluetoothLeScanner inScanner = BluetoothAdapter.getDefaultAdapter().getBluetoothLeScanner();
                if (inScanner != null) {
                    inScanner.stopScan(mScanCallback);
                }
                onIntervalScanUpdate(true);
                mLog.d("Scan ble thread is interrupted");
            });
        } else {
            Toast.makeText(this, "请重启", Toast.LENGTH_SHORT).show();
        }

    }

    private void stopScan() {
        BluetoothAdapter adapter = BluetoothAdapter.getDefaultAdapter();
        BluetoothLeScanner scanner = adapter.getBluetoothLeScanner();
        if (scanner != null) {
            scanner.stopScan(mScanCallback);
        }
        if (mUpdateFuture != null) {
            mUpdateFuture.cancel(true);
        }
        mLog.d("Stop scan ble");
    }

    private void onIntervalScanUpdate(boolean over) {
        List<ScanResult> devices = new ArrayList<>(mDeviceMap.values());
        Collections.sort(devices, (dev1, dev2) -> {
            Integer rssi1 = dev1.getRssi();
            Integer rssi2 = dev2.getRssi();
            return rssi2.compareTo(rssi1);
        });
        runOnUiThread(() -> {
            mBleList.clear();
            mBleList.addAll(devices);
            mBleAdapter.notifyDataSetChanged();

            if (over) {
                mRefreshLayout.setRefreshing(false);
            }
        });
    }

    private void gotoDevice(BluetoothDevice device) {
        Intent intent = new Intent(MainActivity.this, BleActivity.class);
        intent.putExtra(BleConstants.KEY_BLE_DEVICE, device);
        startActivityForResult(intent, REQUEST_BLUFI);

        mDeviceMap.clear();
        mBleList.clear();
        mBleAdapter.notifyDataSetChanged();
    }

    private class BleHolder extends RecyclerView.ViewHolder implements View.OnClickListener {
        ScanResult scanResult;
        TextView text1;
        TextView text2;

        BleHolder(View itemView) {
            super(itemView);

            text1 = itemView.findViewById(android.R.id.text1);
            text2 = itemView.findViewById(android.R.id.text2);

            itemView.setOnClickListener(this);
        }

        @Override
        public void onClick(View v) {
            stopScan();
            PgDialog pd = PgDialog.newInstance(MainActivity.this);
            if (!pd.isShowing()) {
                pd.show("准备连接中...");//显示加载框
            }
            handler.postDelayed(new Runnable() {
                @Override
                public void run() {
                    pd.dismiss();
                    gotoDevice(scanResult.getDevice());
                }
            }, 1000);
        }
    }

    private class ScanCallback extends android.bluetooth.le.ScanCallback {

        @Override
        public void onScanFailed(int errorCode) {
            super.onScanFailed(errorCode);
        }

        @Override
        public void onBatchScanResults(List<ScanResult> results) {
            for (ScanResult result : results) {
                onLeScan(result);
            }
        }

        @Override
        public void onScanResult(int callbackType, ScanResult result) {
            onLeScan(result);
        }

        private void onLeScan(ScanResult scanResult) {
            String name = scanResult.getDevice().getName();
            //Log.d("onLeScan","name:"+name+scanResult.getDevice().getAddress());

            if (!TextUtils.isEmpty(mBlufiFilter)) {
                // delete || !name.startsWith(mBlufiFilter)
                if (name == null) {
                    return;
                }
            }

            mDeviceMap.put(scanResult.getDevice().getAddress(), scanResult);
            if (name != null) {
                if (name.contains(mBlufiFilter)) {
                    stopScan();
                    //mThreadPool.shutdownNow();
                }
            }

        }
    }

    private class BleAdapter extends RecyclerView.Adapter<BleHolder> {

        @NonNull
        @Override
        public BleHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
            View view = getLayoutInflater().inflate(R.layout.main_ble_item, parent, false);
            return new BleHolder(view);
        }

        @Override
        public void onBindViewHolder(@NonNull BleHolder holder, int position) {
            ScanResult scanResult = mBleList.get(position);
            holder.scanResult = scanResult;

            BluetoothDevice device = scanResult.getDevice();
            String name = device.getName() == null ? getString(R.string.string_unknown) : device.getName();
            holder.text1.setText(name);

            SpannableStringBuilder info = new SpannableStringBuilder();
            info.append("Mac:").append(device.getAddress())
                    .append(" RSSI:").append(String.valueOf(scanResult.getRssi()));
            info.setSpan(new ForegroundColorSpan(0xFF9E9E9E), 0, 21, Spannable.SPAN_EXCLUSIVE_INCLUSIVE);
            info.setSpan(new ForegroundColorSpan(0xFF8D6E63), 21, info.length(),
                    Spannable.SPAN_EXCLUSIVE_INCLUSIVE);
            holder.text2.setText(info);
        }

        @Override
        public int getItemCount() {
            return mBleList.size();
        }
    }

    private boolean postContainData()
            throws InterruptedException {
        byte[] data = "qwertyuiopasdfghjklzxcvbnm1234567890!@#$%^&*().............000000".getBytes();
        ByteArrayInputStream dataIS = new ByteArrayInputStream(data);
        ByteArrayOutputStream postOS = new ByteArrayOutputStream();
//        int pkgLengthLimit = mPackageLengthLimit > 0 ? mPackageLengthLimit :
//                (mBlufiMTU > 0 ? mBlufiMTU : DEFAULT_PACKAGE_LENGTH);
//        int postDataLengthLimit = pkgLengthLimit - PACKAGE_HEADER_LENGTH;
//        postDataLengthLimit -= 2; // if flag, two bytes total length in data
//        if (checksum) {
//            postDataLengthLimit -= 2;
//        }
        byte[] dateBuf = new byte[14];
        while (true) {
            int read = dataIS.read(dateBuf, 0, dateBuf.length);
            if (read == -1) {
                break;
            }

            postOS.write(dateBuf, 0, read);
            if (dataIS.available() == 2) {
                postOS.write(dataIS.read());
                postOS.write(dataIS.read());
            }
            boolean frag = dataIS.available() > 0;
            //int frameCtrl = FrameCtrlData.getFrameCTRLValue(encrypt, checksum, DIRECTION_OUTPUT,
            // requireAck, frag);
            //int sequence = generateSendSequence();
            if (frag) {
                int totalLen = postOS.size() + dataIS.available();
                byte[] tempData = postOS.toByteArray();
                postOS.reset();
                postOS.write(totalLen & 0xff);
                postOS.write(totalLen >> 8 & 0xff);
                postOS.write(tempData, 0, tempData.length);
            }
            Log.d("NewGetPostBytes2", "start");

            //byte[] postBytes = NewGetPostBytes2(type, frameCtrl, sequence, postOS.size(), postOS
            // .toByteArray());
            postOS.reset();


        }

        return true;
    }
}