package com.blzn.blefi.ui;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.bluetooth.BluetoothGatt;
import android.bluetooth.BluetoothManager;
import android.bluetooth.BluetoothProfile;
import android.content.Intent;
import android.graphics.Color;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.os.Environment;
import android.os.Handler;
import android.util.Log;
import android.view.MotionEvent;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.CompoundButton;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.ProgressBar;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.RequiresApi;
import androidx.appcompat.app.AppCompatActivity;

import com.blzn.ble.R;
import com.blzn.blefi.app.BaseActivity;
import com.blzn.blefi.constants.BleUUIDParameter;
import com.blzn.blefi.constants.SettingsConstants;
import com.blzn.blefi.manager.ConfigManager;
import com.blzn.blefi.utils.DownloadUtil;
import com.example.hellojni.BlufiApp;
import com.github.lzyzsd.circleprogress.DonutProgress;
import com.google.android.material.checkbox.MaterialCheckBox;

import java.io.File;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.Arrays;
import java.util.Locale;
import java.util.zip.CRC32;

import blufi.ble.BleClient;
import blufi.ble.cb.BusinessCallback;
import blufi.ble.cb.BleException;
import blufi.ble.cb.BleNotifyCallback;
import blufi.ble.cb.BleWriteCallback;

import static com.blzn.blefi.utils.FileUtil.File2byte;
import static com.blzn.blefi.utils.FileUtil.getPath;
import static com.blzn.blefi.utils.HexUtil.formatHexString;

/**
 * @author zhubinsheng
 */
public class OtaActivity extends BaseActivity {
    private static final int FILE_SELECTOR_CODE = 1;
    private static final String TAG = "OtaActivity";
    private Handler handler;

    private DonutProgress donut_progress;
    private TextView localFileDetails;
    private TextView browseFileDetails;
    private ProgressBar pb;

    private ListView infoList;

    private MaterialCheckBox internet_file;
    private MaterialCheckBox local_file;

    private EditText browseFileAddress;
    private EditText crc;

    private byte[] internet_file_bs;
    private byte[] local_file_bs;

    private boolean sendIng = false;
    private boolean sendSuc = false;
    private boolean otaSuc = false;

    private String uuidPreference =
            (String) BlufiApp.getInstance().settingsGet(SettingsConstants.PREF_SETTINGS_KEY_SERVICE_UUID,
                    BleUUIDParameter.UUID_SERVICE.toString());

    private CRC32 c = new CRC32();

    private Runnable reconnect_runnable = new Runnable() {
        @Override
        public void run() {
            BleClient.getInstance().connect();
            handler.postDelayed(reconnect_runnable, 3000);
        }
    };
    private ConfigManager mConfigManager;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        handler = new Handler();
        setContentView(R.layout.activity_ota);
        setHomeAsUpEnable(true);

        donut_progress = findViewById(R.id.donut_progress);
        localFileDetails = findViewById(R.id.localFileDetails);
        browseFileDetails = findViewById(R.id.browseFileDetails);
        internet_file = findViewById(R.id.internet_file);
        local_file = findViewById(R.id.local_file);
        browseFileAddress = findViewById(R.id.browseFileAddress);
        crc = findViewById(R.id.crc);
        infoList = findViewById(R.id.info_list);
        pb = findViewById(R.id.pb);

        BleClient.getInstance().setBusinessCallback(new OtaCallback());
        mConfigManager = ConfigManager.getInstance();
        setOtaFile(mConfigManager.getOtaFilePath(), false);
        setDonut_progress(0, "ready");

        internet_file.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(CompoundButton compoundButton, boolean b) {
                if (b) {
                    local_file.setChecked(false);
                } else {
                    local_file.setChecked(true);
                }
            }
        });
        local_file.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(CompoundButton compoundButton, boolean b) {
                if (b) {
                    internet_file.setChecked(false);
                } else {
                    internet_file.setChecked(true);
                }
            }
        });

        BluetoothManager ble_manager = (BluetoothManager) getSystemService(BLUETOOTH_SERVICE);
        ble_manager.getConnectionState(BleClient.getInstance().getGatt().getDevice(), 8);
        setDonut_progress(1, "设备连接正常");

        BleClient.getInstance().notify(
                BleClient.getInstance().getGatt().getDevice(),
                BleUUIDParameter.getUUID().get(uuidPreference).getUUID_SERVICE().toString(),
                BleUUIDParameter.getUUID().get(uuidPreference).getUUID_NOTIFICATION_CHARACTERISTIC().toString(),
                bleNotifyCallback);
        bleNotifyCallback.setKey(BleUUIDParameter.getUUID().get(uuidPreference).getUUID_NOTIFICATION_CHARACTERISTIC().toString());

    }

    private void setDonut_progress(float progress, String showText) {
        donut_progress.setProgress(progress);
        donut_progress.setText(showText);
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        //add
        BleClient.getInstance().stopNotify(
                BleClient.getInstance().getGatt().getDevice(),
                BleUUIDParameter.getUUID().get(uuidPreference).getUUID_SERVICE().toString(),
                BleUUIDParameter.getUUID().get(uuidPreference).getUUID_NOTIFICATION_CHARACTERISTIC().toString());
        handler.removeCallbacks(reconnect_runnable);
    }

    public void Choose_local_file(View view) {
        Intent intent = new Intent(Intent.ACTION_GET_CONTENT);
        intent.setType("*/*");
        intent.addCategory(Intent.CATEGORY_OPENABLE);
        startActivityForResult(intent, FILE_SELECTOR_CODE);
    }

    public void down_from_browse(View view) {
        String url = browseFileAddress.getText().toString();
        if (url.isEmpty()) {
            Toast.makeText(this, "url is null", Toast.LENGTH_LONG).show();
            return;
        }
        // 下载前应该检查存储中是否已经有这个文件 2覆盖最新版
        DownloadUtil.getInstance().download(url,
                String.valueOf(new File(Environment.getExternalStorageDirectory(),
                        "bl_ota_" + System.currentTimeMillis() + ".bin")),
                new DownloadUtil.DownloadListener() {
                    @Override
                    public void onDownloadSuccess(String path, byte[] bytes) {
                        //回调 传入file对象
                        Log.e(TAG, String.valueOf(bytes.length));
                        Log.w(TAG, "升级文件准备完成:");
                        internet_file_bs = bytes;
                        runOnUiThread(() -> {
                            browseFileDetails.setText("fileSize:" + bytes.length);
                            setDonut_progress(10, "升级文件准备完成");
                            internet_file.setChecked(true);
                            local_file.setChecked(false);
                            Toast.makeText(OtaActivity.this, "Download Success", Toast.LENGTH_LONG);
                        });
                    }

                    @Override
                    public void onDownloading(int progress) {
//                        Log.w(TAG,"progress:"+progress);
//                        pb.setProgress(progress);
                        setDonut_progress(Math.round(progress / 10), "准备升级文件中");
                    }

                    @Override
                    public void onDownloadFailed() {
                        runOnUiThread(() -> Toast.makeText(OtaActivity.this, "Download Failed",
                                Toast.LENGTH_LONG));
                    }
                }
        );
    }

    public void send_image_identity(View view) {
        byte[] bin_bs;
        if (internet_file.isChecked()) {
            bin_bs = internet_file_bs;
        } else {
            bin_bs = local_file_bs;
        }
        if (bin_bs == null || bin_bs.length == 0) {
            Toast.makeText(this, "bin is null", Toast.LENGTH_SHORT).show();
            return;
        }

        c.reset();//Resets CRC-32 to initial value.
        c.update(bin_bs, 0, bin_bs.length);//将数据丢入CRC32解码器
        int value = (int) c.getValue();//获取CRC32 的值  默认返回值类型为long 用于保证返回值是一个正数
        byte[] res = int2Bytes(value);
        for (int a = 0; a < res.length; a++) {
        }
        ByteBuffer recvBuffer = ByteBuffer.allocate(15);
        recvBuffer.order(ByteOrder.BIG_ENDIAN);
        recvBuffer.put((byte) 0);
        recvBuffer.putShort((short) 44);
        recvBuffer.putInt(22);
        recvBuffer.put(intToBytes(bin_bs.length));
        recvBuffer.put(res);
        Log.e("send_image_identity", Arrays.toString(recvBuffer.array()));
        setDonut_progress(15, "升级信息发送中");
        writeOtaBin(recvBuffer.array());
    }

    public void send_ver_req(View view) {
        ByteBuffer recvBuffer = ByteBuffer.allocate(7);
        recvBuffer.order(ByteOrder.BIG_ENDIAN);
        recvBuffer.put((byte) 0x04);
        recvBuffer.putShort((short) 44);
        recvBuffer.putInt(22);
        Log.e("send_image_identity", Arrays.toString(recvBuffer.array()));
        writeOtaBin(recvBuffer.array());
    }

    public void send_bin(int file_offset) {
        byte[] bin_bs;
        if (internet_file.isChecked()) {
            bin_bs = internet_file_bs;
        } else {
            bin_bs = local_file_bs;
        }
        if (bin_bs == null) {
            return;
        }
        runOnUiThread(() -> setDonut_progress(Math.round(20 + (((double) file_offset) / (bin_bs.length)) * 70), "发送中"));

        int fra_length = BleClient.getInstance().getMtu() - (1 + 1 + 6 + 4 + 1);
        ByteBuffer recvBuffer = ByteBuffer.allocate(BleClient.getInstance().getMtu());
        recvBuffer.order(ByteOrder.BIG_ENDIAN);
        recvBuffer.put((byte) 0x02);
        recvBuffer.put((byte) 0);//status

        recvBuffer.putShort((short) 44);
        recvBuffer.putInt(22);

        recvBuffer.put(intToBytes(file_offset));
        if (file_offset + fra_length > bin_bs.length) {
            int left = bin_bs.length - file_offset;
            recvBuffer.put((byte) (left));//data_size
            recvBuffer.put(subBytes(bin_bs, file_offset, left));
            writeOtaBin(subBytes(recvBuffer.array(), 0, 1 + 1 + 6 + 4 + 1 + left));
        } else {
            recvBuffer.put((byte) fra_length);//data_size
            recvBuffer.put(subBytes(bin_bs, file_offset, fra_length));
            Log.e(TAG, "file_offset: " + file_offset + "  " + Arrays.toString(recvBuffer.array()));
            writeOtaBin(recvBuffer.array());
        }
        sendIng = true;
    }

    @Override

    public boolean onTouchEvent(MotionEvent event) {
        Log.d(TAG, "onTouchEvent sendIng:" + sendIng);
        if (sendIng) {
            return true;
        }
        return false;
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (requestCode == FILE_SELECTOR_CODE && resultCode == Activity.RESULT_OK) {
            Uri uri = data.getData();
            String otaFilePath = getPath(this, uri); // Paul Burke写的函数，根据Uri获得文件路径
            setOtaFile(otaFilePath, true);
        }
    }

    private void setOtaFile(String path, boolean isChoose) {
        try {
            File file = new File(path);
            localFileDetails.setText("fileName:" + file.getName());
            byte[] bytes = File2byte(file);
            localFileDetails.setText(localFileDetails.getText().toString() + "\n" + "fileSize:" + bytes.length);
            local_file_bs = bytes;
            internet_file.setChecked(false);
            local_file.setChecked(true);
            setDonut_progress(10, "升级文件准备完成");
            if (isChoose) mConfigManager.setOtaFilePath(path);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private class OtaCallback extends BusinessCallback {
        @Override
        public void onError(BleClient client, int errCode) {

        }

        @SuppressLint("ResourceType")
        @RequiresApi(api = Build.VERSION_CODES.M)
        @Override
        public void onConnectionStateChange(int status, int newState) {
            Log.d(TAG, String.format(Locale.ENGLISH, "onConnectionStateChange  status=%d, newState=%d",
                    status, newState));
            runOnUiThread(() -> {
                if (status == BluetoothGatt.GATT_SUCCESS) {
                    switch (newState) {
                        case BluetoothProfile.STATE_CONNECTED:
                            Log.d(TAG, donut_progress.getText());
                            donut_progress.setTextColor(Color.GREEN);
                            handler.removeCallbacks(reconnect_runnable);

                            if (sendSuc) {
                                donut_progress.setTextColor(Color.BLUE);
                                setDonut_progress(100, "升级完毕");
                                otaSuc = true;
                            }

                            if (donut_progress.getText().equals("断线重连中")) {
                                setDonut_progress(donut_progress.getProgress(), "重连成功");
                                handler.postDelayed(new Runnable() {
                                    @Override
                                    public void run() {
                                        send_image_identity(donut_progress);
                                    }
                                }, 2000);
                            } else if (donut_progress.getText().equals("gatt close")) {
                                setDonut_progress(1, "设备连接正常");
                            }
                            break;
                        case BluetoothProfile.STATE_DISCONNECTED:
                            donut_progress.setText("Bluetooth DISCONNECTED");
                            break;
                    }
                } else {
//                    donut_progress.setText("gatt close");
//                    donut_progress.setTextColor(Color.RED);
                    if (otaSuc) return;
                    if (sendSuc) {
                        donut_progress.setTextColor(Color.GREEN);
                        setDonut_progress(100, "发送完成");
                    }
                    if ((boolean) BlufiApp.getInstance().settingsGet(SettingsConstants.PREF_SETTINGS_KEY_AUTO_CONNECT, false)) {
                        if (sendIng) setDonut_progress(donut_progress.getProgress(), "断线重连中");
                        handler.postDelayed(reconnect_runnable, 1000);
                    }
                    sendIng = false;
                }
            });
        }
    }

    private void writeOtaBin(byte[] data) {
        BleClient.getInstance().write(
                BleClient.getInstance().getGatt().getDevice(),
                BleUUIDParameter.getUUID().get(uuidPreference).getUUID_SERVICE().toString(),
                BleUUIDParameter.getUUID().get(uuidPreference).getUUID_WRITE_CHARACTERISTIC().toString(),
                data,
                new BleWriteCallback() {

                    @Override
                    public void onWriteSuccess(final int current, final int total,
                                               final byte[] justWrite) {
//                        Log.d(TAG,"onWriteSuccess");
                        runOnUiThread(() -> {
                        });
                    }

                    @Override
                    public void onWriteFailure(final BleException exception) {
                        Log.e(TAG, "onWriteFailure");
                        runOnUiThread(() -> {
                        });
                    }
                });
    }

    private BleNotifyCallback bleNotifyCallback = new BleNotifyCallback() {
        @Override
        public void onNotifySuccess() {
            runOnUiThread(new Runnable() {
                @Override
                public void run() {

                }
            });
        }

        @Override
        public void onNotifyFailure(final BleException exception) {
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                }
            });
        }

        @Override
        public void onCharacteristicChanged(byte[] data) {
            Log.d(TAG, Arrays.toString(data));
            ByteBuffer recvBuffer = ByteBuffer.allocate(data.length);
            recvBuffer.order(ByteOrder.BIG_ENDIAN);
            recvBuffer.put(data);
            switch (data[0]) {
                case 1://oad_block_req_t
                    //如getDouble(2)则表示从下标为2的位置开始读取8个字节的数据
                    short manu_code = recvBuffer.getShort(1);
                    int file_ver = recvBuffer.getInt(3);
                    int file_offset = Integer.reverseBytes(recvBuffer.getInt(7));
                    Log.d(TAG,
                            ">>manu_code:" + manu_code + " >>file_ver:" + file_ver + ">>file_offset:" + file_offset);
                    try {
                        send_bin(file_offset);
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                    break;
                //int i = Integer.reverseBytes(buffer.getInt());
                case 3://oad_upgrd_end_t
                    recvBuffer.get(0);
                    short manu_code2 = recvBuffer.getShort(1);
                    int file_ver2 = recvBuffer.getInt(3);
                    Log.d(TAG,
                            ">>>>>>> oad_upgrd_end_t >>manu_code:" + manu_code2 + "  >>file_ver:" + file_ver2);
                    sendIng = false;
                    sendSuc = true;
                    break;
                case 4:
                    String s = new String(data);
                    System.out.println("Decoded String : " + s);
                    String[] strings = get_ver_time(subBytes(data, 1, data.length - 1));
                    runOnUiThread(new Runnable() {
                        @Override
                        public void run() {
                            infoList.setAdapter(new ArrayAdapter<String>(OtaActivity.this,
                                    android.R.layout.simple_list_item_1, strings));//关联Adapter
                        }
                    });

                    break;
                default:
                    break;
            }
        }
    };

    private String[] get_ver_time(byte[] bytes) {
        String[] strings = new String[3];
        int offset_index = 0;
        for (int i = 0; i < strings.length; i++) {
            byte len = bytes[offset_index];
//            byte[] newBytes = subBytes(bytes,len,bytes.length);
            strings[i] = get_info(subBytes(bytes, offset_index + 1, len));
            offset_index++;
            offset_index += len;

        }
        Log.w(TAG, Arrays.toString(strings));
        return strings;
    }

    private String get_info(byte[] bytes) {
        return new String(bytes);
    }

    public static byte[] subBytes(byte[] src, int begin, int count) {
        byte[] bs = new byte[count];
        System.arraycopy(src, begin, bs, 0, count);
        return bs;
    }

    private byte[] shortToBytes(short value) {
        return ByteBuffer.allocate(2).order(ByteOrder.LITTLE_ENDIAN).putShort(value).array();
    }

    private byte[] intToBytes(int value) {
        return ByteBuffer.allocate(4).order(ByteOrder.LITTLE_ENDIAN).putInt(value).array();
    }

    private byte[] int2Bytes(int num) {
        byte[] bytes = new byte[4];
        //通过移位运算，截取低8位的方式，将int保存到byte数组
        bytes[0] = (byte) (num >>> 24);
        bytes[1] = (byte) (num >>> 16);
        bytes[2] = (byte) (num >>> 8);
        bytes[3] = (byte) num;
        return bytes;
    }

    private byte byteToInteger(Byte b) {
        return (byte) (0xff & b);
    }
}