package com.blzn.blefi.ui;

import android.app.Activity;
import android.content.Intent;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.net.Uri;
import android.nfc.Tag;
import android.os.Bundle;
import android.text.TextUtils;
import android.util.Log;

import androidx.annotation.Nullable;
import androidx.annotation.StringRes;
import androidx.appcompat.app.AlertDialog;
import androidx.preference.EditTextPreference;
import androidx.preference.ListPreference;
import androidx.preference.Preference;
import androidx.preference.PreferenceCategory;
import androidx.preference.PreferenceFragmentCompat;
import androidx.preference.SwitchPreference;

import com.blzn.blefi.app.BaseActivity;

import com.blzn.blefi.constants.BleConstants;
import com.blzn.blefi.constants.BleUUIDParameter;
import com.blzn.blefi.constants.SettingsConstants;
import com.blzn.blefi.task.BleAppReleaseTask;
import com.blzn.ble.R;
import com.blzn.blefi.utils.WindowPermissionCheck;
import com.example.hellojni.BlufiApp;

import java.util.UUID;
import java.util.concurrent.atomic.AtomicReference;

import blufi.ble.BleClient;
import blufi.ble.params.BlufiParameter;
import blufi.ble.params.ServiceUuid;

import static com.blzn.blefi.constants.BleUUIDParameter.UUID_SERVICE;

public class SettingsActivity extends BaseActivity {

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.settings_activity);
        //Toolbar toolbar = findViewById(R.id.toolbar);
        //setSupportActionBar(toolbar);
        setHomeAsUpEnable(true);

        getSupportFragmentManager().beginTransaction().replace(R.id.frame, new SettingsFragment()).commit();
    }

    public static class SettingsFragment extends PreferenceFragmentCompat implements Preference.OnPreferenceChangeListener {
        private static final String KEY_MTU_LENGTH = SettingsConstants.PREF_SETTINGS_KEY_MTU_LENGTH;
        private static final String KEY_BLE_PREFIX = SettingsConstants.PREF_SETTINGS_KEY_BLE_PREFIX;
        private static final String KEY_AUTO_CONNECT = SettingsConstants.PREF_SETTINGS_KEY_AUTO_CONNECT;
        private static final String KEY_SERVICE_UUID = SettingsConstants.PREF_SETTINGS_KEY_SERVICE_UUID;

        private BlufiApp mApp;

        private EditTextPreference mMtuPref;
        private EditTextPreference mBlePrefixPref;
        private ListPreference key_listpreference;
        private SwitchPreference auto_connect_switchPreference;
        private SwitchPreference show_speed_frame;

        private Preference mVersionCheckPref;
        private volatile BleAppReleaseTask.ReleaseInfo mAppLatestRelease;

        private long mAppVersionCode;
        private String mAppVersionName;

        private boolean first = true;

        @Override
        public void onCreatePreferences(Bundle savedInstanceState, String rootKey) {
            setPreferencesFromResource(R.xml.blufi_settings, rootKey);
        }

        @Override
        public void onCreate(Bundle savedInstanceState) {
            super.onCreate(savedInstanceState);

            mApp = BlufiApp.getInstance();
            getVersionInfo();
            findPreference(R.string.settings_version_key).setSummary(mAppVersionName);
            findPreference(R.string.settings_blufi_version_key).setSummary(BleClient.VERSION);

            mMtuPref = findPreference(R.string.settings_mtu_length_key);
            int mtuLen = (int) mApp.settingsGet(KEY_MTU_LENGTH, BleConstants.DEFAULT_MTU_LENGTH);
            mMtuPref.setOnPreferenceChangeListener(this);
            if (mtuLen >= BleConstants.MIN_MTU_LENGTH && mtuLen <= BleConstants.MAX_MTU_LENGTH) {
                mMtuPref.setSummary(String.valueOf(mtuLen));
            }
            PreferenceCategory blufiCategory = findPreference(R.string.settings_category_blufi_key);
//            blufiCategory.removePreference(mMtuPref);

            mBlePrefixPref = findPreference(R.string.settings_ble_prefix_key);
            mBlePrefixPref.setOnPreferenceChangeListener(this);
            String blePrefix = (String) mApp.settingsGet(KEY_BLE_PREFIX, BleConstants.BLUFI_PREFIX);
            mBlePrefixPref.setSummary(blePrefix);

            auto_connect_switchPreference = findPreference(R.string.mistake_touch_mode_setting_summary);
            boolean autoConnect = (boolean) mApp.settingsGet(KEY_AUTO_CONNECT, BleConstants.DEFAULT_AUTO_CONNECT);
            auto_connect_switchPreference.setChecked(autoConnect);
            auto_connect_switchPreference.setOnPreferenceChangeListener((preference, newValue) -> {
                Log.d("onPreferenceChange","preference:"+preference+"  newValue:"+newValue);
                boolean prefix = (boolean) newValue;
                mApp.settingsPut(KEY_AUTO_CONNECT, prefix);
                return true;
            });

            show_speed_frame = findPreference(R.string.auto_connect);
            show_speed_frame.setChecked((boolean) mApp.settingsGet(SettingsConstants.SETTINGS_KEY_SHOW_SPEED_FRAME, false));
            show_speed_frame.setOnPreferenceChangeListener((preference, newValue) -> {
                Log.d("onPreferenceChange","preference:"+preference+"  newValue:"+newValue);
                boolean prefix = (boolean) newValue;
                mApp.settingsPut(SettingsConstants.SETTINGS_KEY_SHOW_SPEED_FRAME, prefix);
                if (prefix){
                    WindowPermissionCheck.checkPermission(getActivity());
                }
                return true;
            });

            key_listpreference = findPreference(R.string.key_listpreference);
            key_listpreference.setOnPreferenceChangeListener(this);
            String listpreference = (String) mApp.settingsGet(KEY_SERVICE_UUID, UUID_SERVICE.toString());
            key_listpreference.setSummary(listpreference);
            key_listpreference.setOnPreferenceClickListener(new Preference.OnPreferenceClickListener() {
                @Override
                public boolean onPreferenceClick(Preference preference) {
                    if (first){
                        if (BlufiApp.getInstance().settingsGet(SettingsConstants.SELECT_SETTINGS_KEY_SERVICE_UUID, UUID_SERVICE.toString()) != null) {
                            if (!BlufiApp.getInstance().settingsGet(SettingsConstants.SELECT_SETTINGS_KEY_SERVICE_UUID, UUID_SERVICE.toString()).equals(UUID_SERVICE.toString())) {
                                setListPreferenceData(key_listpreference, getActivity());
                            }
                        }
                    }

                    return false;
                }
            });

            mVersionCheckPref = findPreference(R.string.settings_upgrade_check_key);

        }

        protected ListPreference setListPreferenceData(ListPreference lp, Activity mActivity) {
            if (lp != null) {
                CharSequence categoryList[] = lp.getEntries();
                CharSequence old_entryValues[]  = lp.getEntryValues();

                CharSequence entries[] = new String[categoryList.length+1];
                CharSequence entryValues[] = new String[categoryList.length+1];
                System.arraycopy(categoryList,0,entries,0,categoryList.length);
                System.arraycopy(old_entryValues,0,entryValues,0,old_entryValues.length);
                entries[entries.length -1 ] = (CharSequence) BlufiApp.getInstance().settingsGet(SettingsConstants.SELECT_SETTINGS_KEY_SERVICE_UUID, UUID_SERVICE.toString());
                entryValues[entryValues.length -1 ] = (CharSequence) BlufiApp.getInstance().settingsGet(SettingsConstants.SELECT_SETTINGS_KEY_SERVICE_UUID, UUID_SERVICE.toString());


                lp.setEntries(entries);
                lp.setEntryValues(entryValues);
            }
            first = false;
            return lp;
        }

        public <T extends Preference> T findPreference(@StringRes int res) {
            return findPreference(getString(res));
        }

        private void getVersionInfo() {
            try {
                PackageInfo pi = requireActivity().getPackageManager()
                        .getPackageInfo(requireActivity().getPackageName(), 0);
                mAppVersionName = pi.versionName;
                mAppVersionCode = pi.versionCode;
            } catch (PackageManager.NameNotFoundException e) {
                mAppVersionName = getString(R.string.string_unknown);
                mAppVersionCode = -1;
            }
        }

        @Override
        public boolean onPreferenceTreeClick(Preference preference) {
            if (preference == mVersionCheckPref) {
                if (mAppLatestRelease == null) {
                    mVersionCheckPref.setEnabled(false);
                    //checkAppLatestRelease();
                } else {
                    //downloadLatestRelease();
                }
                return true;
            }

            return super.onPreferenceTreeClick(preference);
        }

        @Override
        public boolean onPreferenceChange(Preference preference, Object newValue) {
            Log.d("onPreferenceChange","preference:"+preference+"  newValue:"+newValue);
            if (preference == mMtuPref) {
                String lenStr = newValue.toString();
                int mtuLen = BleConstants.DEFAULT_MTU_LENGTH;
                if (!TextUtils.isEmpty(lenStr)) {
                    int newLen = Integer.parseInt(lenStr);
                    mtuLen = Math.min(BleConstants.MAX_MTU_LENGTH, Math.max(BleConstants.MIN_MTU_LENGTH, newLen));
                }
                mMtuPref.setSummary(String.valueOf(mtuLen));
                mApp.settingsPut(KEY_MTU_LENGTH, mtuLen);
                return true;
            } else if (preference == mBlePrefixPref) {
                String prefix = newValue.toString();
                mBlePrefixPref.setSummary(prefix);
                mApp.settingsPut(KEY_BLE_PREFIX, prefix);
                return true;
            }else if (preference == key_listpreference) {
                String prefix = newValue.toString();
                key_listpreference.setSummary(prefix);
                mApp.settingsPut(KEY_SERVICE_UUID, prefix);
                return true;
            }

            return false;
        }

        private void downloadLatestRelease() {
            Intent intent = new Intent(Intent.ACTION_VIEW);
            Uri uri = Uri.parse(mAppLatestRelease.getDownloadUrl());
            intent.setData(uri);
            startActivity(intent);
        }
    }  // Fragment end
} // Activity end
