package com.blzn.blefi.ui.wight;
 
import android.app.Dialog;
import android.content.Context;
import android.content.DialogInterface;
import android.text.method.ScrollingMovementMethod;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup.LayoutParams;
import android.widget.Button;
import android.widget.CompoundButton;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.RadioButton;
import android.widget.RadioGroup;
import android.widget.Switch;
import android.widget.TextView;

import com.blzn.ble.R;

import java.util.Arrays;

public class CustomDialog extends Dialog {
 
	public CustomDialog(Context context) {
		super(context);
	}
 
	public CustomDialog(Context context, int theme) {
		super(context, theme);
	}
 
	public static class Builder {
		private Context context;
		private String positiveButtonText;
		private String negativeButtonText;
		private ClickListener positiveButtonClickListener;
		private DialogInterface.OnClickListener negativeButtonClickListener;

		private EditText text_input;
		private EditText interval;
		private Switch readSwitch;
		private TextView testWriteBytes;
		private TextView speed;
		private TextView readText;
		private TextView acceptSize;
		private TextView acceptSpeed;

		private Button b;

		private RadioButton single;
		private RadioButton continuity;

		private LinearLayout setInterval;

		private String TAG = "CustomDialog";
		private int acceptSizeNum;

		public Builder(Context context) {
			this.context = context;
		}
 
		/**
		 * Set the positive button resource and it's listener
		 * 
		 * @param positiveButtonText
		 * @param listener
		 * @return
		 */
		public Builder setPositiveButton(int positiveButtonText,
										 ClickListener listener) {
			this.positiveButtonText = (String) context
					.getText(positiveButtonText);
			this.positiveButtonClickListener = listener;
			return this;
		}
 
		public Builder setPositiveButton(String positiveButtonText,
										 ClickListener listener) {
			this.positiveButtonText = positiveButtonText;
			this.positiveButtonClickListener = listener;
			return this;
		}
 
		public Builder setNegativeButton(int negativeButtonText,
				DialogInterface.OnClickListener listener) {
			this.negativeButtonText = (String) context
					.getText(negativeButtonText);
			this.negativeButtonClickListener = listener;
			return this;
		}
 
		public Builder setNegativeButton(String negativeButtonText,
				DialogInterface.OnClickListener listener) {
			this.negativeButtonText = negativeButtonText;
			this.negativeButtonClickListener = listener;
			return this;
		}
 
		public CustomDialog create() {
			LayoutInflater inflater = (LayoutInflater) context
					.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
			// instantiate the dialog with the custom Theme
			final CustomDialog dialog = new CustomDialog(context, R.style.Theme_AppCompat_Dialog_Alert);
			dialog.setCancelable(false);
			View layout = inflater.inflate(R.layout.fragment_dialog_write, null);
			testWriteBytes = layout.findViewById(R.id.testWriteBytes);
			speed = layout.findViewById(R.id.speed);
			single = layout.findViewById(R.id.single);
			continuity = layout.findViewById(R.id.continuity);
			setInterval = layout.findViewById(R.id.setInterval);
			text_input = layout.findViewById(R.id.text_input);
			interval = layout.findViewById(R.id.interval);
			interval.setText("10");

			readText = layout.findViewById(R.id.readText);
			acceptSize = layout.findViewById(R.id.acceptSize);
			acceptSpeed = layout.findViewById(R.id.acceptSpeed);
			readText.setMovementMethod(ScrollingMovementMethod.getInstance());
			b = (Button) layout.findViewById(R.id.start);
			readSwitch = layout.findViewById(R.id.readSwitch);
			readSwitch.setOnCheckedChangeListener((buttonView, isChecked) -> positiveButtonClickListener.onRead(isChecked));


			dialog.addContentView(layout, new LayoutParams(
					LayoutParams.MATCH_PARENT, LayoutParams.WRAP_CONTENT));
			((Button) layout.findViewById(R.id.clear)).setOnClickListener(new View.OnClickListener() {
				@Override
				public void onClick(View view) {
					text_input.setText("");
				}
			});
			// set the confirm button
			if (positiveButtonText != null) {
				((Button) layout.findViewById(R.id.start))
						.setText(positiveButtonText);
				if (positiveButtonClickListener != null) {
					((Button) layout.findViewById(R.id.start))
							.setOnClickListener(new View.OnClickListener() {
								public void onClick(View v) {
									if (b.getText().equals("停止")){
										layout.findViewById(R.id.close).setVisibility(
												View.VISIBLE);
										b.setText("发送");
										positiveButtonClickListener.onStop();
									}else {
										if (text_input.getText().toString().isEmpty()){
											return;
										}
										if (single.isChecked()){
											positiveButtonClickListener.onClick(dialog,text_input.getText().toString(),-1,
													DialogInterface.BUTTON_POSITIVE);
										}else {
											positiveButtonClickListener.onClick(dialog,text_input.getText().toString(),Integer.parseInt(interval.getText().toString()),
													DialogInterface.BUTTON_POSITIVE);
											layout.findViewById(R.id.close).setVisibility(
													View.GONE);
											b.setText("停止");
										}

									}

								}
							});
				}
			} else {
				// if no confirm button just set the visibility to GONE
				layout.findViewById(R.id.start).setVisibility(
						View.GONE);
			}
			// set the cancel button
			if (negativeButtonText != null) {
				((Button) layout.findViewById(R.id.close))
						.setText(negativeButtonText);
				if (negativeButtonClickListener != null) {
					((Button) layout.findViewById(R.id.close))
							.setOnClickListener(new View.OnClickListener() {
								public void onClick(View v) {
									negativeButtonClickListener.onClick(dialog,
											DialogInterface.BUTTON_NEGATIVE);
								}
							});
				}
			} else {
				// if no confirm button just set the visibility to GONE
				layout.findViewById(R.id.close).setVisibility(
						View.GONE);
			}

			RadioGroup radgroup = (RadioGroup) layout.findViewById(R.id.radioGroup);
			radgroup.setOnCheckedChangeListener(new RadioGroup.OnCheckedChangeListener() {
				@Override
				public void onCheckedChanged(RadioGroup group, int checkedId) {
					RadioButton radbtn = (RadioButton) layout.findViewById(checkedId);
					if(R.id.single!=checkedId&&radbtn.isChecked()){
						setInterval.setVisibility(View.VISIBLE);
						single.setChecked(false);
						continuity.setChecked(true);
					}
					if(R.id.continuity!=checkedId&&radbtn.isChecked()){
						setInterval.setVisibility(View.GONE);
						continuity.setChecked(false);
						single.setChecked(true);
					}

				}
			});

			dialog.setContentView(layout);
			return dialog;
		}

		public void setWriteBytes(int testWriteBytes) {
			this.testWriteBytes.setText(testWriteBytes+ " Byte");
		}

		public void setSpeed(int num ,int acceptSpeed) {
			this.speed.setText(num+ " B/s");
			this.acceptSpeed.setText(acceptSpeed+ " B/s");
		}

		public void refreshLogView(byte[] date){
//			readText.append(Arrays.toString(date));
//			int offset=readText.getLineCount()*readText.getLineHeight();
//			if(offset>readText.getHeight()){
//				readText.scrollTo(0,offset-readText.getHeight());
//			}
			acceptSizeNum+=date.length;
			this.acceptSize.setText(acceptSizeNum+ " Byte");
		}

	}

	public interface ClickListener {
		void onClick(CustomDialog dialog, String s, int i, int buttonPositive);
		void onStop();
		void onRead(boolean b);
	}
}