package blufi.ble.cb;

import android.bluetooth.BluetoothGatt;
import android.bluetooth.BluetoothGattCharacteristic;
import android.bluetooth.BluetoothGattService;

import blufi.ble.BleClient;
import blufi.ble.response.BleVersionResponse;

public abstract class BleCallback {
    public static final int STATUS_SUCCESS = 0;

    public static final int CODE_INVALID_NOTIFICATION = -1000;
    public static final int CODE_CATCH_EXCEPTION = -1001;
    public static final int CODE_WRITE_DATA_FAILED = -1002;
    public static final int CODE_INVALID_DATA = -1003;

    public static final int CODE_NEG_POST_FAILED = -2000;
    public static final int CODE_NEG_ERR_DEV_KEY = -2001;
    public static final int CODE_NEG_ERR_SECURITY = -2002;
    public static final int CODE_NEG_ERR_SET_SECURITY = -2003;

    public static final int CODE_CONF_INVALID_OPMODE = -3000;
    public static final int CODE_CONF_ERR_SET_OPMODE = -3001;
    public static final int CODE_CONF_ERR_POST_STA = -3002;
    public static final int CODE_CONF_ERR_POST_SOFTAP = -3003;

    /**
     * Callback invoked after BluetoothGattCallback receive onServicesDiscovered
     *
     * @param client BleClient
     * @param gatt BluetoothGatt
     * @param service null if discover Blufi GattService failed
     * @param writeChar null if discover Blufi write GattCharacteristic failed
     * @param notifyChar null if discover Blufi notify GattCharacteristic failed
     */
    public void onGattPrepared(BleClient client, BluetoothGatt gatt, BluetoothGattService service,
                               BluetoothGattCharacteristic writeChar, BluetoothGattCharacteristic notifyChar) {
    }

    /**
     * Callback invoked when receive Gatt notification
     *
     * @param client BleClient
     * @param pkgType Blufi package type
     * @param subType Blufi subtype
     * @param data notified
     * @return true if the callback consumed the notification, false otherwise.
     */
    public boolean onGattNotification(BleClient client, int pkgType, int subType, byte[] data) {
        return false;
    }

    /**
     * Callback invoked when received error code from device
     *
     * @param client BleClient
     * @param errCode received
     */
    public void onError(BleClient client, int errCode) {
    }

    /**
     * Callback invoked when negotiate security over with device
     *
     * @param client BleClient
     * @param status {@link #STATUS_SUCCESS} means negotiate security success
     */
    public void onNegotiateSecurityResult(BleClient client, int status) {
    }

    /**
     * Callback invoked when post config data over
     *
     * @param client BleClient
     * @param status {@link #STATUS_SUCCESS} means post data success
     */
    public void onConfigureResult(BleClient client, int status) {
    }

    /**
     * Callback invoked when received device version
     *
     * @param client BleClient
     * @param status {@link #STATUS_SUCCESS} means response is valid
     * @param response BleVersionResponse
     */
    public void onDeviceVersionResponse(BleClient client, int status, BleVersionResponse response) {
    }

    /**
     * Callback invoked when received device status
     *  @param client BleClient
     * @param status {@link #STATUS_SUCCESS} means response is valid
     * @param wifiState
     */
    public void onDeviceStatusResponse(BleClient client, int status, String wifiState) {
    }

    public void onDeviceScanResult(BleClient client, int status, String results) {
    }

    /**
     * Callback invoked when post custom data over
     *
     * @param client BleClient
     * @param status {@link #STATUS_SUCCESS} means post data success
     * @param data posted
     */
    public void onPostCustomDataResult(BleClient client, int status,String data) {
    }

    /**
     * Callback invoked when received custom data from device
     *
     * @param client BleClient
     * @param status {@link #STATUS_SUCCESS} means receive data success
     * @param data received
     */
    public void onReceiveCustomData(BleClient client, int status, byte[] data) {
    }

    public void onThroughput(int num,int read_data_size) {
    }
}
