package com.example.udplibrary;


import android.annotation.SuppressLint;
import android.app.ActivityManager;
import android.content.Context;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.location.LocationManager;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.net.wifi.ScanResult;
import android.net.wifi.WifiInfo;
import android.net.wifi.WifiManager;
import android.os.Build;
import android.util.Log;

import androidx.annotation.RequiresApi;

import java.util.List;

public class NetStatusUtil {

    /**
     * 判断当前手机是否连上Wifi.
     *
     * @param context
     *            上下文
     * @return boolean 是否连上网络
     *
     *         *
     */
    static public boolean isWifiConnected(Context context) {
        if (context != null) {
            ConnectivityManager mConnectivityManager = (ConnectivityManager) context
                    .getSystemService(Context.CONNECTIVITY_SERVICE);
            @SuppressLint("MissingPermission") NetworkInfo mWiFiNetworkInfo = mConnectivityManager.getNetworkInfo(ConnectivityManager.TYPE_WIFI);
            if (mWiFiNetworkInfo != null) {
                if (mWiFiNetworkInfo.isAvailable()) {
                    return mWiFiNetworkInfo.isConnected();
                } else {
                    return false;
                }
            }
        }
        return false;
    }

    /**
     * 判断当前网络是手机网络还是WIFI.
     *
     * @param context
     *            上下文
     * @return ConnectedType 数据类型
     *
     *         *
     */
    public static int getConnectedType(Context context) {
        if (context != null) {
            ConnectivityManager mConnectivityManager = (ConnectivityManager) context
                    .getSystemService(Context.CONNECTIVITY_SERVICE);
            // 获取代表联网状态的NetWorkInfo对象
            @SuppressLint("MissingPermission") NetworkInfo mNetworkInfo = mConnectivityManager.getActiveNetworkInfo();
            // 判断NetWorkInfo对象是否为空；判断当前的网络连接是否可用
            if (mNetworkInfo != null && mNetworkInfo.isAvailable()) {
                return mNetworkInfo.getType();
            }
        }
        return -1;
    }

    /**
     * 获取当前WIFI的SSID.
     *
     * @param context
     *            上下文
     * @return ssid
     *
     *         *
     */
    @RequiresApi(api = Build.VERSION_CODES.LOLLIPOP)
    public static String getCurentWifiSSID(Context context) {
        String ssid = "";
        if (context != null) {
            WifiManager wifiManager = (WifiManager) context.getApplicationContext().getSystemService(Context.WIFI_SERVICE);
            WifiInfo wifiInfo = wifiManager.getConnectionInfo();
            int frequency = wifiInfo.getFrequency();
            boolean is = is5GHzWifi(frequency);
            if (is){
                return "ture";
            }
            if(wifiInfo!=null){
                ssid = wifiInfo.getSSID();
                Log.d("zbs-SSID2",ssid);
                if (ssid.contains("unknown ssid")){
                    boolean isOpenGps = isOpenGps(context);
                    if (!isOpenGps){
                        return "please OpenGps";
                    }
                    Log.d("zbs-isOpenGps", String.valueOf(isOpenGps));
                }
                if (ssid.substring(0, 1).equals("\"") && ssid.substring(ssid.length() - 1).equals("\"")) {
                    ssid = ssid.substring(1, ssid.length() - 1);
                }
            }

        }


        return ssid;
    }

    /**
     * 判断是否2.4Gwifi
     * @param frequency
     * @return
     */
    public boolean is24GHzWifi(int frequency){
        return frequency > 2400 && frequency < 2500;
    }
    /**
     * 判断是否5Gwifi
     * @param frequency
     * @return
     */
    private static boolean is5GHzWifi(int frequency) {
        return frequency > 4900 && frequency < 5900;
    }

    /**
     * 判断是否打开网络定位
     * @param context 用于获取LocationManager
     * @return true/false
     */
    public static boolean isOpenGps(Context context){
        LocationManager locationManager = (LocationManager)context.getSystemService(Context.LOCATION_SERVICE);
        if (locationManager != null){
            //这里只判断了是否开启网络定位
            //还有GPS定位            LocationManager.GPS_PROVIDER
            return locationManager.isProviderEnabled(LocationManager.NETWORK_PROVIDER);
        }else {
            return false;
        }
    }

    /**
     * 用来获得手机扫描到的所有wifi的信息.
     *
     * @param c
     *            上下文
     * @return the current wifi scan result
     */
    static public List<ScanResult> getCurrentWifiScanResult(Context c) {
        WifiManager wifiManager = (WifiManager) c.getSystemService(Context.WIFI_SERVICE);
        wifiManager.startScan();
        return wifiManager.getScanResults();
    }

    static public String getConnectWifiSsid(Context c) {
        String ssid = "";
        WifiManager wifiManager = (WifiManager) c.getSystemService(Context.WIFI_SERVICE);
        WifiInfo wifiInfo = wifiManager.getConnectionInfo();
        if(wifiInfo!=null){
            ssid = wifiInfo.getSSID();
        }
        return ssid;
    }

    // 以下是获得版本信息的工具方法

    // 版本名
    public static String getVersionName(Context context) {
        return getPackageInfo(context).versionName;
    }

    // 版本号
    public static int getVersionCode(Context context) {
        return getPackageInfo(context).versionCode;
    }

    private static PackageInfo getPackageInfo(Context context) {
        PackageInfo pi = null;

        try {
            PackageManager pm = context.getPackageManager();
            pi = pm.getPackageInfo(context.getPackageName(), PackageManager.GET_CONFIGURATIONS);

            return pi;
        } catch (Exception e) {
            e.printStackTrace();
        }

        return pi;
    }

    // 检测android 应用在前台还是后台

    public static boolean isBackground(Context context) {
        ActivityManager activityManager = (ActivityManager) context.getSystemService(Context.ACTIVITY_SERVICE);
        List<ActivityManager.RunningAppProcessInfo> appProcesses = activityManager.getRunningAppProcesses();
        for (ActivityManager.RunningAppProcessInfo appProcess : appProcesses) {
            if (appProcess.processName.equals(context.getPackageName())) {
				/*
				 * BACKGROUND=400 EMPTY=500 FOREGROUND=100 GONE=1000
				 * PERCEPTIBLE=130 SERVICE=300 ISIBLE=200
				 */
                Log.i(context.getPackageName(), "此appimportace =" + appProcess.importance
                        + ",context.getClass().getName()=" + context.getClass().getName());
                if (appProcess.importance != ActivityManager.RunningAppProcessInfo.IMPORTANCE_FOREGROUND) {
                    Log.i(context.getPackageName(), "处于后台" + appProcess.processName);
                    return true;
                } else {
                    Log.i(context.getPackageName(), "处于前台" + appProcess.processName);
                    return false;
                }
            }
        }
        return false;
    }
}
