package com.example.udplibrary;

import android.os.Handler;
import android.util.Log;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.SocketException;
import java.net.UnknownHostException;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.Arrays;
import java.util.Timer;
import java.util.TimerTask;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;
import java.util.zip.CRC32;

public class UDPClient implements Runnable {
    private final Handler handler;
    private ExecutorService mThreadPool;

    private int udpPort = 9999;
    private String hostIp = "192.168.1.4";
    private static DatagramSocket socket = null;
    private static DatagramPacket packetSend, packetRcv;
    private boolean udpLife = true; //udp生命线程
    private byte[] msgRcv = new byte[1024]; //接收消息
    private CRC32 c = new CRC32();
    private String TAG = "UDPClient";
    private Callback callback;
    private byte seq = 0;
    private byte last_send_seq = 0;
    private InetAddress hostAddress = null;
    private byte[] _msgSend;
    private boolean _needAck;
    private Timer timer = new Timer();

    private int send_error_time = 0;
    //心跳包
    private TimerTask task = new TimerTask() {
        @Override
        public void run() {
            ByteBuffer sendBuffer = ByteBuffer.allocate(2);
            sendBuffer.order(ByteOrder.BIG_ENDIAN);
            sendBuffer.position(0);
            sendBuffer.put((byte) 0x77);
            sendBuffer.put((byte) 0x77);

//            send(sendBuffer.array(),false);
        }
    };
    private Lock mWriteLock = new ReentrantLock(true);

    //发送消息
    Runnable TimeoutRunnable = () -> {
        synchronized (mWriteLock) {
            Log.i("udpClient", "TimeoutRunnable111");
            mWriteLock.notifyAll();
            Log.i("udpClient", "TimeoutRunnable222");
            if (send_error_time > 3) {
                callback.onSendDFail();
                return;
            }
            mThreadPool.submit(() -> {
                send(_msgSend, _needAck);
            });
            send_error_time++;
        }
    };
    private short MAGIC_NUM = 123;

    public byte[] intToBytes(int value) {
        return ByteBuffer.allocate(4).order(ByteOrder.BIG_ENDIAN).putInt(value).array();
    }

    public UDPClient(Handler handler, String hostIp, int udpPort, Callback callback) {
        super();
        this.hostIp = hostIp;
        this.udpPort = udpPort;
        this.callback = callback;
        this.handler = handler;
        mThreadPool = Executors.newSingleThreadExecutor();

        try {
            hostAddress = InetAddress.getByName(hostIp);
        } catch (UnknownHostException e) {
            Log.i("udpClient", "未找到服务器");
            e.printStackTrace();
        }
    }

    //返回udp生命线程因子是否存活
    public boolean isUdpLife() {
        if (udpLife) {
            return true;
        }

        return false;
    }

    //更改UDP生命线程因子
    public void setUdpLife(boolean b) {
        udpLife = b;
    }

    public void auto_connect(boolean isChecked) {
        short length = 8;
        ByteBuffer sendBuffer = ByteBuffer.allocate(length);
        sendBuffer.order(ByteOrder.LITTLE_ENDIAN);
        sendBuffer.position(0);

        sendBuffer.putShort(MAGIC_NUM);
        sendBuffer.putShort(length);

        sendBuffer.put(seq);

        if (isChecked) {
            sendBuffer.put((byte) BlufiParameter.Type.Ctrl.CMD_STA_AUTOCONNECT_ENABLE);
        } else {
            sendBuffer.put((byte) BlufiParameter.Type.Ctrl.CMD_STA_AUTOCONNECT_DISABLE);
        }

        sendBuffer.putShort((short) 0);

        mThreadPool.submit(() -> {
            send(sendBuffer.array(), true);
        });

    }

    public void wifi_list_get() {
        short length = 8;
        ByteBuffer sendBuffer = ByteBuffer.allocate(length);
        sendBuffer.order(ByteOrder.LITTLE_ENDIAN);
        sendBuffer.position(0);
        sendBuffer.putShort(MAGIC_NUM);
        sendBuffer.putShort(length);
        sendBuffer.put(seq);
        sendBuffer.put((byte) BlufiParameter.Type.Ctrl.SUBTYPE_GET_WIFI_LIST);
        sendBuffer.putShort((short) 0);
        mThreadPool.submit(() -> {
            send(sendBuffer.array(), true);
        });
    }

    public void disconnect_wifi() {
        short length = 8;
        ByteBuffer sendBuffer = ByteBuffer.allocate(length);
        sendBuffer.order(ByteOrder.LITTLE_ENDIAN);
        sendBuffer.position(0);
        sendBuffer.putShort(MAGIC_NUM);
        sendBuffer.putShort(length);
        sendBuffer.put(seq);
        sendBuffer.put((byte) BlufiParameter.Type.Ctrl.SUBTYPE_DISCONNECT_WIFI);
        sendBuffer.putShort((short) 0);
        mThreadPool.submit(() -> {
            send(sendBuffer.array(), true);
        });
    }

    public void sendWifiInfo(String json) {
        byte[] jsonBytes = json.getBytes();
        c.reset();//Resets CRC-32 to initial value.
        c.update(jsonBytes);

        ByteBuffer sendBuffer = ByteBuffer.allocate(jsonBytes.length + 7 * 4);
        sendBuffer.order(ByteOrder.LITTLE_ENDIAN);
        sendBuffer.position(0);

        sendBuffer.putShort(MAGIC_NUM);
        sendBuffer.putShort((short) (jsonBytes.length + 7 * 4));

        sendBuffer.put(seq);
        sendBuffer.put((byte) BlufiParameter.Type.Data.SUBTYPE_STA_WIFI_SSID);

        sendBuffer.putShort((short) 0);
        sendBuffer.putInt(1);
        sendBuffer.putInt(jsonBytes.length);
        sendBuffer.putInt(1);
        sendBuffer.putInt(jsonBytes.length);
        sendBuffer.put(intToBytes((int) c.getValue()));

        sendBuffer.put(jsonBytes);

        mThreadPool.submit(() -> {
            send(sendBuffer.array(), true);
        });
    }

    public void getStatus() {
        short length = 8;
        ByteBuffer sendBuffer = ByteBuffer.allocate(length);
        sendBuffer.order(ByteOrder.LITTLE_ENDIAN);
        sendBuffer.position(0);

        sendBuffer.putShort(MAGIC_NUM);
        sendBuffer.putShort(length);

        sendBuffer.put(seq);
        sendBuffer.put((byte) BlufiParameter.Type.Ctrl.SUBTYPE_GET_WIFI_STATUS);
        sendBuffer.putShort((short) 0);

        mThreadPool.submit(() -> {
            send(sendBuffer.array(), true);
        });
    }

    public synchronized void send(byte[] msgSend, boolean needAck) {
        Log.w(TAG + " send:", Arrays.toString(msgSend));
        this._msgSend = msgSend;
        this._needAck = needAck;

        packetSend = new DatagramPacket(msgSend, msgSend.length, hostAddress, udpPort);
        mWriteLock.lock();
        synchronized (mWriteLock) {
            try {
                try {
                    socket.send(packetSend);
                } catch (IOException e) {
                    e.printStackTrace();
                }

                if (needAck) {
                    handler.postDelayed(TimeoutRunnable, 1234);
                    mWriteLock.wait(1334);
                }
            } catch (InterruptedException e) {
                e.printStackTrace();
                Log.w("udpClient", "发送失败");
            } finally {
                mWriteLock.unlock();
            }
        }
    }

    public void send_ack(byte _seq) {
        short length = 8;
        ByteBuffer sendBuffer = ByteBuffer.allocate(length);
        sendBuffer.order(ByteOrder.LITTLE_ENDIAN);
        sendBuffer.position(0);
        sendBuffer.putShort(MAGIC_NUM);
        sendBuffer.putShort(length);
        sendBuffer.put(_seq);
        sendBuffer.put((byte) BlufiParameter.Type.Ctrl.SUBTYPE_ACK);
        sendBuffer.putShort((short) 1);
        send(sendBuffer.array(), false);
    }

    @Override
    public void run() {
        try {
            socket = new DatagramSocket();
//            socket.setSoTimeout(3000);//设置超时为3s
        } catch (SocketException e) {
            Log.i("udpClient", "建立接收数据报失败");
            e.printStackTrace();
        }
        packetRcv = new DatagramPacket(msgRcv, msgRcv.length);

        timer.schedule(task, 100, 3000);                //启动定时器

        while (udpLife) {
            try {
                socket.receive(packetRcv);
                String RcvMsg = new String(packetRcv.getData(), packetRcv.getOffset(), packetRcv.getLength());
                if (RcvMsg.getBytes()[0] == 0x77 && RcvMsg.getBytes()[1] == 0x77) {
                    Log.i("Rcv", "心跳包");
                    if (callback != null) {
                        callback.onConnectionStateChange(1, 1);
                    }
                    continue;
                }
                Log.i("udpClient", "RcvMsg.getBytes()[4]:" + RcvMsg.getBytes()[4]);
                Log.i("udpClient", "RcvMsg.getBytes()[7]:" + RcvMsg.getBytes()[7]);
                Log.i("udpClient", "RcvMsg.getBytes()[6]:" + RcvMsg.getBytes()[6]);
                Log.i("udpClient", "last_send_seq:" + last_send_seq);

                if (RcvMsg.getBytes()[6] == 0) {
                    Log.i("udpClient", "receive data");
                    send_ack(RcvMsg.getBytes()[4]);

                    switch (RcvMsg.getBytes()[5]) {
                        case BlufiParameter.Type.Data.SUBTYPE_WIFI_LIST:
                            Log.i("udpClient", "SUBTYPE_WIFI_LIST ");
                            Log.w(TAG, new String(packetRcv.getData(), 28, packetRcv.getLength() - 28));
                            try {
                                if (callback != null) {
                                    callback.onWifiScan(new String(packetRcv.getData(), 28,
                                            packetRcv.getLength() - 28));
                                }
                            } catch (Exception e) {
                                Log.e(TAG, "gson fromJson fail");
                            }
                            break;

                        case BlufiParameter.Type.Data.SUBTYPE_WIFI_CONNECTION_STATE:
                            String RcvMsg2 = new String(packetRcv.getData(), 28, packetRcv.getLength() - 28);
                            Log.w(TAG, RcvMsg2);
                            if (callback != null) {
                                callback.onWifiStatus(RcvMsg2);
                            }
                            break;
                    }
                } else {
                    Log.i("udpClient", "receive ack");

                    if (RcvMsg.getBytes()[4] == seq) {
                        Log.i("udpClient", "onSendSuccess");
                        if (_needAck) {
                            handler.removeCallbacks(TimeoutRunnable);
                        }
                        synchronized (mWriteLock) {
                            mWriteLock.notify();
                        }
                        callback.onSendSuccess();
                        send_error_time = 0;
                        last_send_seq = seq;
                        seq++;
                    }
                }

                //将收到的消息发给主界面
//                Intent RcvIntent = new Intent();
//                RcvIntent.setAction("udpRcvMsg");
//                RcvIntent.putExtra("udpRcvMsg", RcvMsg);
//                WifiApActivity.context.sendBroadcast(RcvIntent);
//
                Log.i("Rcv", Arrays.toString(RcvMsg.getBytes()));
            } catch (IOException e) {
                e.printStackTrace();
            }
        }

        Log.i("udpClient", "UDP监听关闭");
        socket.close();
        timer.cancel();
    }

    public interface Callback {
        void onConnectionStateChange(int status, int newState);

        void onSendSuccess();

        void onWifiStatus(String wifiState);

        void onWifiScan(String wifiInfo);

        void onSendDFail();
    }
}