package com.example.wifiapp;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.app.Dialog;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.net.wifi.ScanResult;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.text.TextUtils;
import android.util.Log;
import android.view.KeyEvent;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.Window;
import android.view.animation.AlphaAnimation;
import android.view.animation.Animation;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.Spinner;
import android.widget.Switch;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.RequiresApi;
import androidx.appcompat.app.AlertDialog;

import org.json.JSONException;
import org.json.JSONObject;

import com.example.udplibrary.NetStatusUtil;
import com.example.udplibrary.UDPClient;
import com.example.udplibrary.WifiInfo;
import com.example.udplibrary.WifiUtils;
import com.example.udplibrary.wifiState;
import com.example.udplibrary.BlbleParameter.Type.*;
import com.google.gson.Gson;

import java.util.ArrayList;
import java.util.List;

public class WifiApActivity extends Activity implements OnClickListener{
	private static final String TAG = WifiApActivity.class.getSimpleName();
	private TextView tvTitle;
	private Spinner device_wifi_list;
	private ImageView ivArrow;
	private EditText etSsid, etPwd;
	private Button btnConfig;
	private Button connect_status;

	private boolean hasSend = false;

	private final int BREATH_INTERVAL_TIME = 500; //设置呼吸灯时间间隔
	private AlphaAnimation animationFadeIn;
	private AlphaAnimation animationFadeOut;

	private static final String SERVER_IP = "192.168.169.1";
	private static final int SERVER_PORT = 5002;

	private ProgressDialog loadingDialog;
	private TextView wifi_status;
	private String[] wifiList = new String[50];

	private Handler handler = new Handler();
	private ArrayAdapter<String> _Adapter;

	private Runnable runnable = new Runnable() {
		@SuppressLint("ResourceAsColor")
		@Override
		public void run() {
			connect_status.setText("失去信号");
			connect_status.setBackgroundColor(R.color.gray);
		}
	};
	private UDPClient client = new UDPClient(handler,SERVER_IP, SERVER_PORT , new UDPClient.Callback() {
		@Override
		public void onConnectionStateChange(int status, int newState) {
			runOnUiThread(new Runnable() {
				@SuppressLint("ResourceAsColor")
				@Override
				public void run() {
					connect_status.setText("已经连接");
					connect_status.setBackgroundColor(R.color.green);
					connect_status.startAnimation(animationFadeOut);
					handler.removeCallbacks(runnable);
					handler.postDelayed(runnable,3000);
				}
			});
		}

		@Override
		public void onSendSuccess() {

		}

		@Override
		public void onWifiStatus(String wifiStateStr) {
			hasSend = false;
			Gson gson = new Gson();
			wifiState wifiState = gson.fromJson(wifiStateStr, wifiState.class);
			switch (wifiState.getState()){
//				case WIFI_STATE_ENUM_LIST
//						.WIFI_STATE_CONNECTED_IP_GOT:
				case WIFI_STATE_ENUM_LIST
						.WIFI_STATE_WITH_AP_CONNECTED_IP_GOT:

					runOnUiThread(new Runnable() {
						@Override
						public void run() {
							AlertDialog alertDialog2 = new AlertDialog.Builder(WifiApActivity.this)
									.setTitle("这是标题")
									.setMessage(wifiState.toString())
									.setPositiveButton("确定", new DialogInterface.OnClickListener() {//添加"Yes"按钮
										@Override
										public void onClick(DialogInterface dialogInterface, int i) {
										}
									})

									.setNegativeButton("取消", new DialogInterface.OnClickListener() {//添加取消
										@Override
										public void onClick(DialogInterface dialogInterface, int i) {
										}
									})
									.setNeutralButton("断开WIFI", new DialogInterface.OnClickListener() {//添加普通按钮
										@Override
										public void onClick(DialogInterface dialogInterface, int i) {
										}
									})
									.create();
							alertDialog2.show();
						}
					});

					updateMessage("WIFI CONNECTED IP: " + wifiState.getIp()
							, true);
					break;
				case WIFI_STATE_ENUM_LIST
						.WIFI_STATE_CONNECTING:

					Toast.makeText(WifiApActivity.this,"wifi连接中",Toast.LENGTH_SHORT).show();
					handler.postDelayed(new Runnable() {
						@Override
						public void run() {
							requestDeviceStatus();
						}
					},500);
					break;

				case WIFI_STATE_ENUM_LIST
						.WIFI_STATE_IDLE:
					case WIFI_STATE_ENUM_LIST
							.WIFI_STATE_WITH_AP_IDLE:
				case WIFI_STATE_ENUM_LIST
						.WIFI_STATE_WITH_AP_DISCONNECT:
					updateMessage("设备wifi断开,status code: "+wifiState.getState(), true);
					break;

				case WIFI_STATE_ENUM_LIST
						.WIFI_STATE_CONNECTED_IP_GETTING:
					Toast.makeText(WifiApActivity.this,"设备获取ip地址中",Toast.LENGTH_SHORT).show();

					handler.postDelayed(new Runnable() {
						@Override
						public void run() {
							requestDeviceStatus();
						}
					},300);
					break;

				case WIFI_STATE_ENUM_LIST
						.WIFI_STATE_PSK_ERROR:
					updateMessage("设备wifi密码错误", true);
					break;

				case WIFI_STATE_ENUM_LIST
						.WIFI_STATE_NO_AP_FOUND:
					updateMessage("设备未找到该wifi", true);
					break;

				default:
					updateMessage("未知状态，请稍后手动更新,status code: "+wifiState.getState(), true);
					break;

			}

		}

		@Override
		public void onWifiScan(String wifiInfoStr) {
			Gson gson = new Gson();
			WifiInfo wifiInfo = gson.fromJson(wifiInfoStr, WifiInfo.class);
			Log.e("getSsidudpClient",wifiInfo.getSsid());
			if (wifiInfo.getSsid().isEmpty()){
				return;
			}
			runOnUiThread(new Runnable() {
				@Override
				public void run() {
					_Adapter.add(wifiInfo.getSsid());
					_Adapter.notifyDataSetChanged();
				}
			});
		}

		@Override
		public void onSendDFail() {

		}
	});

	private void requestDeviceStatus() {
	}

	private void updateMessage(String s, boolean b) {
		runOnUiThread(() -> wifi_status.setText(s));
	}

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		// TODO Auto-generated method stub
		super.onCreate(savedInstanceState);
		requestWindowFeature(Window.FEATURE_NO_TITLE);
		setContentView(R.layout.activity_main);
		findView();
		initListener();
		initUI();
		List<String> start = new ArrayList<>();
		_Adapter=new ArrayAdapter<String>(this,android.R.layout.simple_spinner_item, start);
		device_wifi_list.setAdapter(_Adapter);
		device_wifi_list.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
			@Override
			public void onItemSelected(AdapterView<?> parent, View view,
									   int position, long id) {
				String str=parent.getItemAtPosition(position).toString();
				etSsid.setText(str);
			}
			@Override
			public void onNothingSelected(AdapterView<?> parent) {
				// TODO Auto-generated method stub
			}
		});

		new Thread(client).start();
		WifiUtils utils = WifiUtils.getInstance(this);

		if (utils.isWifiEnable()){
			Log.d("initIntent","WifiEnable");
		}else {
			Log.d("initIntent","WifiE_unable");
			utils.openWifi();
		}
//		utils.connectWifiPws(getWifiname(), getWifipassword(),getActivity());
	}

	private void findView() {
		// TODO Auto-generated method stub
		device_wifi_list = findViewById(R.id.device_wifi_list);
		tvTitle = (TextView) findViewById(R.id.tv_title);
		ivArrow = (ImageView) findViewById(R.id.iv_arrow);
		etSsid = (EditText) findViewById(R.id.et_ssid);
		etPwd = (EditText) findViewById(R.id.et_pwd);
		btnConfig = (Button) findViewById(R.id.btn_config);
		connect_status = (Button) findViewById(R.id.connect_status);
		wifi_status = (TextView) findViewById(R.id.wifi_status);

	}


	private void initListener() {
		// TODO Auto-generated method stub
		tvTitle.setOnClickListener(this);
		btnConfig.setOnClickListener(this);
	}

	private void initUI() {
		// TODO Auto-generated method stub
		tvTitle.setText(R.string.device_name_z400twb);
//		initConfig(deviceType);
		etSsid.setText("bl_test_");
		etPwd.setText("12345678");
		
		loadingDialog = new ProgressDialog(this);
		loadingDialog.setCancelable(false);
		loadingDialog.setCanceledOnTouchOutside(false);
		loadingDialog.setMessage(getString(R.string.loading_pair_wifi));


		Switch auto_connect = findViewById(R.id.auto_connect);
		auto_connect.setOnCheckedChangeListener((buttonView, isChecked) -> client.auto_connect(isChecked));

		animationFadeIn = new AlphaAnimation(0.1f, 1.0f);
		animationFadeIn.setDuration(BREATH_INTERVAL_TIME);
//        animationFadeIn.setStartOffset(100);

		animationFadeOut = new AlphaAnimation(1.0f, 0.1f);
		animationFadeOut.setDuration(BREATH_INTERVAL_TIME);
//        animationFadeIn.setStartOffset(100);

		animationFadeIn.setAnimationListener(new Animation.AnimationListener(){

			@Override
			public void onAnimationEnd(Animation arg0) {
				connect_status.startAnimation(animationFadeOut);
			}

			@Override
			public void onAnimationRepeat(Animation arg0) {
				// TODO Auto-generated method stub

			}

			@Override
			public void onAnimationStart(Animation arg0) {
				// TODO Auto-generated method stub

			}

		});

		animationFadeOut.setAnimationListener(new Animation.AnimationListener(){

			@Override
			public void onAnimationEnd(Animation arg0) {
				connect_status.startAnimation(animationFadeIn);
			}

			@Override
			public void onAnimationRepeat(Animation arg0) {
				// TODO Auto-generated method stub

			}

			@Override
			public void onAnimationStart(Animation arg0) {
				// TODO Auto-generated method stub

			}

		});
	}

	@Override
	public void onClick(View v) {
		// TODO Auto-generated method stub
		if(v == tvTitle){
			new SelectDeviceDialog(this).show();
		}else if(v == btnConfig){
			final String ssid = etSsid.getText().toString();
			final String pwd = etPwd.getText().toString();

			if(TextUtils.isEmpty(ssid)){
				Toast.makeText(this, R.string.input_wifi_name, Toast.LENGTH_SHORT).show();
				return;
			}
			
			loadingDialog.show();

			new Thread(new Runnable() {
				@Override
				public void run() {
					JSONObject jsonObject=new  JSONObject();
					try {
						jsonObject.put("ssid",ssid);
						jsonObject.put("pwd",pwd);
					} catch (JSONException e) {
						e.printStackTrace();
					}
					String json=jsonObject.toString();
					client.sendWifiInfo(json);
				}
			}).start();

			hasSend = true;
			handler.postDelayed(new Runnable() {
				@Override
				public void run() {
					if (hasSend){
						btn_status_get(loadingDialog.getCurrentFocus());
					}
				}
			},2000);

			handler.postDelayed(new Runnable() {
				@Override
				public void run() {
					if (loadingDialog.isShowing()){
						loadingDialog.dismiss();
					}
				}
			},5000);

		}
	}
	
	@RequiresApi(api = Build.VERSION_CODES.JELLY_BEAN_MR1)
	public boolean isActivityAlive(Activity activity) {
        if (activity == null || activity.isFinishing() || activity.isDestroyed()) {
            return false;
        }
        return true;
    }
	
	
	@Override
	public boolean onKeyDown(int keyCode, KeyEvent event) {
		// TODO Auto-generated method stub
//		if(keyCode == KeyEvent.KEYCODE_BACK){
//			return true;
//		}
		return super.onKeyDown(keyCode, event);
	}
	
	
	@Override
	protected void onDestroy() {
		// TODO Auto-generated method stub\
		client.setUdpLife(false);
		super.onDestroy();
	}

	public void btn_status_get(View view) {
		client.getStatus();
	}

	public void btn_disconnect_wifi(View view) {
		client.disconnect_wifi();
	}

	public void wifi_list_get(View view) {
		_Adapter.clear();
		client.wifi_list_get();
	}

	private void requestDeviceWifiScan() {
		Intent intent = new Intent();
		startActivity(intent);
	}

	private String[] paycenterIdIntArray;

	public void phone_wifi_list(View view) {
		List<ScanResult>  scanResults =  NetStatusUtil.getCurrentWifiScanResult(this);
		paycenterIdIntArray = new String[scanResults.size()];
		for (int i = 0; i < scanResults.size(); i++) {
			ScanResult scanResults1 = scanResults.get(i);
			paycenterIdIntArray[i] = scanResults1.SSID;
		}

		showSingleAlertDialog(view);
	}


	private AlertDialog alertDialog2; //单选框
	public void showSingleAlertDialog(View view){
		AlertDialog.Builder alertBuilder = new AlertDialog.Builder(this);
		alertBuilder.setTitle("select wifi");
		alertBuilder.setSingleChoiceItems(paycenterIdIntArray, 0, new DialogInterface.OnClickListener() {
			@Override
			public void onClick(DialogInterface dialogInterface, int i) {
				etSsid.setText(paycenterIdIntArray[i]);
				alertDialog2.dismiss();
			}
		});

		alertBuilder.setPositiveButton("确定", new DialogInterface.OnClickListener() {
			@Override
			public void onClick(DialogInterface dialogInterface, int i) {
				alertDialog2.dismiss();
			}
		});

		alertBuilder.setNegativeButton("取消", new DialogInterface.OnClickListener() {
			@Override
			public void onClick(DialogInterface dialogInterface, int i) {
				alertDialog2.dismiss();
			}
		});

		alertDialog2 = alertBuilder.create();
		alertDialog2.show();
	}

	class SelectDeviceDialog extends Dialog implements OnClickListener {

		public SelectDeviceDialog(Context context) {
			super(context, R.style.selectDeviceDialog);
			// TODO Auto-generated constructor stub
		}
		
		@Override
		protected void onCreate(Bundle savedInstanceState) {
			// TODO Auto-generated method stub
			super.onCreate(savedInstanceState);
	        setCancelable(true);
		}
		
		@Override
		public void onClick(View v) {
			// TODO Auto-generated method stub

			dismiss();
		}
		
	}
	
}








































